/*
 * @(#)AboutDialog.java	0.1 02/10/03 SMI
 *
 * Copyright (c) 2002-2005 Martin Dubuc All Rights Reserved.
 *
 * Martin Dubuc grants you ("Licensee") a non-exclusive, royalty free, license
 * to use, modify and redistribute this software in source and binary
 * code form, provided that i) this copyright notice and license appear
 * on all copies of the software; and ii) Licensee does not utilize the
 * software in a manner which is disparaging to Martin Dubuc.
 *
 * This software is provided "AS IS," without a warranty of any kind.
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. MARTIN DUBUC AND
 * ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE
 * SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL MARTIN DUBUC OR ITS LICENSORS
 * BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING
 * OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF MARTIN DUBUC HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control
 * of aircraft, air traffic, aircraft navigation or aircraft
 * communications; or in the design, construction, operation or
 * maintenance of any nuclear facility. Licensee represents and
 * warrants that it will not use or redistribute the Software for such
 * purposes.
 */

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import javax.swing.*;

/**
A single response modal about dialog. When the OK button is pressed the dialog returns.
*/
public class AboutDialog extends JDialog implements ActionListener {
  final static long serialVersionUID = 0;
  
  final private int WIDTH = 350;
  final private int HEIGHT = 150;
  final private String BOOTLOADER_LINK = "http://mdubuc.freeshell.org/Jolt";
  /**
  Creates a new <code>AboutDialog</code>.

  @param parent Any JFrame.
  */
  public AboutDialog(JFrame parent) {
    super(parent, BootloaderResourceBundle.getString("ABOUT_TEXT") + " " + Bootloader.PRODUCT_NAME, true);

    Container contentPane = getContentPane();

    JPanel labelPanel = new JPanel();
    labelPanel.setLayout(new GridLayout(3, 1));
    JLabel productLabel = new JLabel(Bootloader.PRODUCT_NAME + " v1.4 (2007)", JLabel.CENTER);
    labelPanel.add(productLabel);
    JLabel authorLabel = new JLabel(BootloaderResourceBundle.getString("BY_TEXT") + " Martin Dubuc (mdubuc@freeshell.org)", JLabel.CENTER);
    labelPanel.add(authorLabel);

    linkLabel = new JLabel(BOOTLOADER_LINK, JLabel.CENTER);
    
    if (!System.getProperty("os.name").startsWith("Mac")) {
      linkLabel.setForeground(Color.BLUE);
      
      linkLabel.addMouseListener(new MouseListener() {
      	public void mouseClicked(MouseEvent e) {
      	  launchBrowser(BOOTLOADER_LINK);
      	} // mouseClicked
      	
		public void mouseEntered(MouseEvent e) {
		  setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		} // mouseEntered
		
		public void mouseExited(MouseEvent e) {
		  setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        } // mouseExited
		
      	public void mousePressed(MouseEvent e) {} // mousePressed
		public void mouseReleased(MouseEvent e) {} // mouseReleased
      });
    } // AboutDialog

    labelPanel.add(linkLabel);

    contentPane.add(labelPanel, "Center");

    JPanel buttonPanel = new JPanel();
    JButton okButton = new JButton(BootloaderResourceBundle.getString("OK_TEXT"));
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    contentPane.add(buttonPanel, "South");

    setSize(WIDTH, HEIGHT);
    setLocation(parent.getLocationOnScreen().x + 30, 
		        parent.getLocationOnScreen().y + 30);

	// Calculate width of link label text         
	width = getFontMetrics(linkLabel.getFont()).stringWidth(linkLabel.getText());

	setResizable(false);
	setVisible(true);
    
    repaint();
  } // AboutDialog

  /**
  Handles events from the OK button. When OK is pressed the dialog becomes
  invisible, disposes of its self, and retruns.
  */
  public void actionPerformed(ActionEvent e) {
    setVisible(false);
    dispose();
  } // actionPerformed
    
  public void launchBrowser(String url) {
    if (System.getProperty("os.name").startsWith("Windows"))
      try {
    	Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + url);
      }
      catch(IOException e) {
    }
    else if (!System.getProperty("os.name").startsWith("Mac")) {
	  boolean foundBrowser = false;
	  try {
	    Process p = Runtime.getRuntime().exec("which mozilla");
	    if (p.waitFor() == 0) {
		  Runtime.getRuntime().exec("mozilla " + url);
		  foundBrowser = true;
	    }
	  }
	  catch(IOException e) {
	  }
	  catch(InterruptedException e) {
	  }
      
	  if (!foundBrowser) {
	    try {        
		  Process p = Runtime.getRuntime().exec("which netscape");
		  if (p.waitFor() == 0) {
		    Runtime.getRuntime().exec("netscape " + url);
		    foundBrowser = true;
		  }
	    }
	    catch(IOException e) {
	    }
	    catch(InterruptedException e) {
	    }
	  }
        
	  if (!foundBrowser) {
	    try {
		  Process p = Runtime.getRuntime().exec("which xterm");
		  if (p.waitFor() == 0) {
		    p = Runtime.getRuntime().exec("which lynx");
		    if (p.waitFor() == 0) {
			  Runtime.getRuntime().exec("xterm -e lynx " + url);
			  foundBrowser = true;
		    }
		  }
	    }
	    catch(IOException e) {
	    }
	    catch(InterruptedException e) {
	    }
	  }
    }
  } // launchBrowser
  
  public void paint(Graphics g) {
    super.paint(g);

	if (System.getProperty("os.name").startsWith("Mac"))
	  return;

    // Draw the underline under the link
    g.setColor(Color.BLUE);
      
	// Calculate position of link label text
	Point pos = linkLabel.getLocation();
    
	int x = pos.x + (getWidth() - width) / 2;
	int y = pos.y + linkLabel.getVerticalTextPosition() +
	getFontMetrics(linkLabel.getFont()).getAscent() + 30;
	
	if (System.getProperty("os.name").startsWith("Windows XP"))
      y += 6;

    g.drawLine(x, y + 1, x + width, y + 1);
  } // paint

  private JLabel linkLabel;
  private int width;
} // AboutDialog
