/*
 * @(#)AlertDialog.java	0.1 02/10/03 SMI
 *
 * Copyright (c) 2002 Martin Dubuc All Rights Reserved.
 *
 * Martin Dubuc grants you ("Licensee") a non-exclusive, royalty free, license
 * to use, modify and redistribute this software in source and binary
 * code form, provided that i) this copyright notice and license appear
 * on all copies of the software; and ii) Licensee does not utilize the
 * software in a manner which is disparaging to Martin Dubuc.
 *
 * This software is provided "AS IS," without a warranty of any kind.
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. MARTIN DUBUC AND
 * ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE
 * SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL MARTIN DUBUC OR ITS LICENSORS
 * BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING
 * OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF MARTIN DUBUC HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control
 * of aircraft, air traffic, aircraft navigation or aircraft
 * communications; or in the design, construction, operation or
 * maintenance of any nuclear facility. Licensee represents and
 * warrants that it will not use or redistribute the Software for such
 * purposes.
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
A single response modal alert dialog. This class is configurable for message 
and title. The width of the dialog will be longer than the longest message 
line.  When the OK button is pressed the dialog returns.
*/
public class AlertDialog extends JDialog implements ActionListener {
  final static long serialVersionUID = 0;

  /**
  Creates a new <code>AlertDialog</code> with three lines of message and
  a title.

  @param parent Any JFrame.
  @param title The title to appear in the border of the dialog.
  @param lineOne The first line of the message in the dialog.
  @param lineTwo The second line of the message in the dialog.
  @param lineThree The third line of the message in the dialog.
  */
  public AlertDialog(JFrame parent, 
                     String title, 
                     String lineOne, 
                     String lineTwo,
                     String lineThree) {
    super(parent, title, true);

    Container contentPane = getContentPane();

    JPanel labelPanel = new JPanel();
    labelPanel.setLayout(new GridLayout(3, 1));
    labelPanel.add(new JLabel(lineOne, JLabel.CENTER));
    labelPanel.add(new JLabel(lineTwo, JLabel.CENTER));
    labelPanel.add(new JLabel(lineThree, JLabel.CENTER));
    contentPane.add(labelPanel, "Center");

    JPanel buttonPanel = new JPanel();
    JButton okButton = new JButton(BootloaderResourceBundle.getString("OK_TEXT"));
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    contentPane.add(buttonPanel, "South");

    FontMetrics fm = getFontMetrics(getFont());
    int width = Math.max(fm.stringWidth(lineOne), 
                         Math.max(fm.stringWidth(lineTwo), fm.stringWidth(lineThree)));

    setSize(width + 40, 150);
    setLocation(parent.getLocationOnScreen().x + 30, 
		        parent.getLocationOnScreen().y + 30);
    setVisible(true);
  }

  /**
  Handles events from the OK button. When OK is pressed the dialog becomes
  invisible, disposes of its self, and retruns.
  */
  public void actionPerformed(ActionEvent e) {
    setVisible(false);
    dispose();
  } // actionPerformed
} // AlertDialog
