/*
 * @(#)ConfigDialog.java	0.1 02/10/03 SMI
 *
 * Copyright (c) 2002, 2003 Martin Dubuc All Rights Reserved.
 *
 * Martin Dubuc grants you ("Licensee") a non-exclusive, royalty free, license
 * to use, modify and redistribute this software in source and binary
 * code form, provided that i) this copyright notice and license appear
 * on all copies of the software; and ii) Licensee does not utilize the
 * software in a manner which is disparaging to Martin Dubuc.
 *
 * This software is provided "AS IS," without a warranty of any kind.
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. MARTIN DUBUC AND
 * ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE
 * SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL MARTIN DUBUC OR ITS LICENSORS
 * BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING
 * OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF MARTIN DUBUC HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control
 * of aircraft, air traffic, aircraft navigation or aircraft
 * communications; or in the design, construction, operation or
 * maintenance of any nuclear facility. Licensee represents and
 * warrants that it will not use or redistribute the Software for such
 * purposes.
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
A single response modal about dialog. When the OK button is pressed the dialog returns.
*/
public class ConfigDialog extends JDialog implements ActionListener {
  final static long serialVersionUID = 0;

  final private int WIDTH = 750;
  final private int HEIGHT = 550;

  final private static int CONFIG1L = 0;
  final private static int CONFIG1H = 1;
  final private static int CONFIG2L = 2;
  final private static int CONFIG2H = 3;
  final private static int CONFIG3L = 4;
  final private static int CONFIG3H = 5;
  final private static int CONFIG4L = 6;
  final private static int CONFIG4H = 7;
  final private static int CONFIG5L = 8;
  final private static int CONFIG5H = 9;
  final private static int CONFIG6L = 10;
  final private static int CONFIG6H = 11;
  final private static int CONFIG7L = 12;
  final private static int CONFIG7H = 13;

  private JCheckBox usbpllCheckBox;
  private Choice cpudivChoice;
  private Choice plldivChoice;
  private JCheckBox iesoCheckBox;
  private JCheckBox fcmenCheckBox;
  private JCheckBox oscsenCheckBox;
  private Choice foscChoice;
  private JCheckBox vregenCheckBox;
  private Choice borvChoice;
  private JCheckBox bodenCheckBox;
  private Choice borenChoice;
  private JCheckBox pwrtenCheckBox;
  private JCheckBox winenCheckBox;
  private Choice wdtpChoice;
  private JCheckBox wdtenCheckBox;
  private JCheckBox waitCheckBox;
  private JCheckBox bwCheckBox;
  private Choice abwChoice;
  private JCheckBox gptrenCheckBox;
  private JCheckBox nott1oscmxCheckBox;
  private JCheckBox hpolCheckBox;
  private JCheckBox lpolCheckBox;
  private JCheckBox pwmpinCheckBox;
  private Choice pmChoice;
  private JCheckBox mclreCheckBox;
  private JCheckBox exclkmxCheckBox;
  private JCheckBox pwm4mxCheckBox;
  private JCheckBox sspmxCheckBox;
  private JCheckBox lpt1oscCheckBox;
  private JCheckBox eccpmxCheckBox;
  private JCheckBox t1oscmxCheckBox;
  private JCheckBox pbadenCheckBox;
  private JCheckBox fltamxCheckBox;
  private JCheckBox ccp2mxCheckBox;
  private JCheckBox debugCheckBox;
  private JCheckBox xinstCheckBox;
  private Choice bbsizChoice;
  private JCheckBox icprtCheckBox;
  private JCheckBox lvpCheckBox;
  private JCheckBox stvrenCheckBox;
  private JCheckBox cpCheckBox;
  private JCheckBox cp0CheckBox;
  private JCheckBox cp1CheckBox;
  private JCheckBox cp2CheckBox;
  private JCheckBox cp3CheckBox;
  private JCheckBox cp4CheckBox;
  private JCheckBox cp5CheckBox;
  private JCheckBox cp6CheckBox;
  private JCheckBox cp7CheckBox;
  private JCheckBox cpbCheckBox;
  private JCheckBox cpdCheckBox;
  private JCheckBox wrt0CheckBox;
  private JCheckBox wrt1CheckBox;
  private JCheckBox wrt2CheckBox;
  private JCheckBox wrt3CheckBox;
  private JCheckBox wrt4CheckBox;
  private JCheckBox wrt5CheckBox;
  private JCheckBox wrt6CheckBox;
  private JCheckBox wrt7CheckBox;
  private JCheckBox wrtcCheckBox;
  private JCheckBox wrtbCheckBox;
  private JCheckBox wrtdCheckBox;
  private JCheckBox ebtrCheckBox;
  private JCheckBox ebtr0CheckBox;
  private JCheckBox ebtr1CheckBox;
  private JCheckBox ebtr2CheckBox;
  private JCheckBox ebtr3CheckBox;
  private JCheckBox ebtr4CheckBox;
  private JCheckBox ebtr5CheckBox;
  private JCheckBox ebtr6CheckBox;
  private JCheckBox ebtr7CheckBox;
  private JCheckBox ebtrbCheckBox;

  private Bootloader bootloader;
  private PicDevice device;
  private byte config[];

  /**
  Creates a new <code>ConfigDialog</code>.

  @param parent Any JFrame.
  @param connection The serial connection.
  */
  public ConfigDialog(Bootloader bootloader, SerialConnection connection) {
    super(bootloader, true);

    int i;

    Container contentPane = getContentPane();

    JPanel configPanel = new JPanel();

    configPanel.setLayout(new GridLayout(8, 1));

    this.bootloader = bootloader;

    device = bootloader.getDevice();

    config = new byte[device.hiConfigMem - device.loConfigMem + 1];
    for (i = device.loConfigMem; i <= device.hiConfigMem; i++)
      config[i - device.loConfigMem] = bootloader.getConfig(i);

    Config1Panel config1Panel = new Config1Panel();
  
    Config2Panel config2Panel = new Config2Panel();
 
	Config3LPanel config3LPanel = new Config3LPanel();
	
	Config3HPanel config3HPanel = new Config3HPanel();

	Config4LPanel config4LPanel = new Config4LPanel();
	
    Config5Panel config5Panel = new Config5Panel();

    Config6Panel config6Panel = new Config6Panel();

    Config7Panel config7Panel = new Config7Panel();

    config1Panel.selectConfig();
    config2Panel.selectConfig();
	config3LPanel.selectConfig();
	config3HPanel.selectConfig();
	config4LPanel.selectConfig();
    config5Panel.selectConfig();
    config6Panel.selectConfig();
    config7Panel.selectConfig();

    configPanel.add(config1Panel);
    
    configPanel.add(config2Panel);
    
	if (device.getConfigBits().isWait() || device.getConfigBits().isBw() ||
		device.getConfigBits().isNott1oscmx() || 
		device.getConfigBits().isHpol() || device.getConfigBits().isLpol() || 
		device.getConfigBits().isPwmpin() || device.getConfigBits().isPm()) {
	  configPanel.add(config3LPanel);
	}
	
	if (device.getConfigBits().isMclre() || device.getConfigBits().isExclkmx() ||
		device.getConfigBits().isPwm4mx() || device.getConfigBits().isSspmx() ||
		device.getConfigBits().isLpt1osc() ||
		device.getConfigBits().isPbaden() || device.getConfigBits().isT1oscmx() ||
		device.getConfigBits().isEccpmx() || device.getConfigBits().isFltamx() ||
		device.getConfigBits().isCcp2mx()) {
	  configPanel.add(config3HPanel);
	}
	
    configPanel.add(config4LPanel);

    configPanel.add(config5Panel);
    
    if (device.getConfigBits().isWrt0() ||
    	device.getConfigBits().isWrtb()) {
      configPanel.add(config6Panel);
    }
    
    configPanel.add(config7Panel);

    contentPane.add(configPanel, "Center");

    JPanel buttonPanel = new JPanel();
    JButton okButton = new JButton(BootloaderResourceBundle.getString("OK_TEXT"));
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    JButton cancelButton = new JButton(BootloaderResourceBundle.getString("CANCEL_TEXT"));
    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);
    contentPane.add(buttonPanel, "South");

    FontMetrics fm = getFontMetrics(getFont());

    setSize(WIDTH, HEIGHT);

    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    setLocation(screenSize.width / 2 - WIDTH / 2, 
                screenSize.height / 2 - HEIGHT / 2);
                
    setTitle(BootloaderResourceBundle.getString("EDIT_CONFIGURATION_TEXT"));
             
    setVisible(true);
  } // ConfigDialog

  private void setConfiguration() {
  	int mask;
 
	if (usbpllCheckBox != null && usbpllCheckBox.isSelected()) {
	  config[CONFIG1L] |= 1 << 5;
	}
	else {
	  config[CONFIG1L] &= (~(1 << 5)) & 0xff;
	}

    if (device.getConfigBits().isCpudiv()) {
      config[CONFIG1L] &= (~0x18) & 0xff;
      config[CONFIG1L] |= cpudivChoice.getSelectedIndex() << 3;
    }

    if (device.getConfigBits().isPlldiv()) {
      config[CONFIG1L] &= (~7) & 0xff;
      config[CONFIG1L] |= plldivChoice.getSelectedIndex();
    }

	if (iesoCheckBox != null && iesoCheckBox.isSelected()) {
	  config[CONFIG1H] |= 1 << 7;
	}
	else {
	  config[CONFIG1H] &= (~(1 << 7)) & 0xff;
	}

	if (fcmenCheckBox != null && fcmenCheckBox.isSelected()) {
		config[CONFIG1H] |= 1 << 6;
	}
	else {
		config[CONFIG1H] &= (~(1 << 6)) & 0xff;
	}

    if (oscsenCheckBox != null && oscsenCheckBox.isSelected()) {
      config[CONFIG1H] |= 1 << 5;
    }
    else {
      config[CONFIG1H] &= (~(1 << 5)) & 0xff;
    }

    if (device.getConfigBits().isFosc20() || device.getConfigBits().isFosc30()) {
      if (device.getConfigBits().isFosc30())
        mask = 0xf;
      else
        mask = 7;
      config[CONFIG1H] &= (~mask) & 0xff;
      config[CONFIG1H] |= foscChoice.getSelectedIndex();
    }

    if (vregenCheckBox != null && vregenCheckBox.isSelected()) {
      config[CONFIG2L] |= 1 << 5;
    }
    else {
      config[CONFIG2L] &= (~(1 << 5)) & 0xff;
    }

    if (device.getConfigBits().isBoren10()) {
      config[CONFIG2L] &= (~0x18) & 0xff;
      config[CONFIG2L] |= borvChoice.getSelectedIndex() << 3;
    }
    else {
      config[CONFIG2L] &= (~0x0c) & 0xff;
      config[CONFIG2L] |= borvChoice.getSelectedIndex() << 2;
    }
    
    if (bodenCheckBox != null && bodenCheckBox.isSelected()) {
      config[CONFIG2L] |= 1 << 1;
    }
    else {
      config[CONFIG2L] &= (~(1 << 1)) & 0xff;
    }

    if (borenChoice != null) {
      config[CONFIG2L] &= (~6) & 0xff;
      config[CONFIG2L] |= borenChoice.getSelectedIndex() << 1;
    }

    if (pwrtenCheckBox != null && pwrtenCheckBox.isSelected()) {
      config[CONFIG2L] |= 1;
    }
    else {
      config[CONFIG2L] &= 0xfe;
    }

	if (winenCheckBox != null && winenCheckBox.isSelected()) {
	  config[CONFIG2H] |= 1 << 5;
	}
	else {
	  config[CONFIG2H] &= (~(1 << 5)) & 0xff;
	}

	if (wdtpChoice != null) {
	  if (device.getConfigBits().isWdtps30())
        config[CONFIG2H] &= (~0x1e) & 0xff;
	  else
	    config[CONFIG2H] &= (~0x0e) & 0xff;
      config[CONFIG2H] |= wdtpChoice.getSelectedIndex() << 1;
	}
	
    if (wdtenCheckBox != null && wdtenCheckBox.isSelected()) {
      config[CONFIG2H] |= 1;
    }
    else {
      config[CONFIG2H] &= 0xfe;
    }

	if (waitCheckBox != null && waitCheckBox.isSelected()) {
      config[CONFIG3L] |= 1 << 7;
	}
	else {
	  config[CONFIG3L] &= (~(1 << 7)) & 0xff;
	}

	if (bwCheckBox != null && bwCheckBox.isSelected()) {
	  config[CONFIG3L] |= 1 << 6;
	}
	else {
	  config[CONFIG3L] &= (~(1 << 6)) & 0xff;
	}

    if (abwChoice != null) {
        config[CONFIG3L] &= ~(3 << 4) & 0xff;
        config[CONFIG3L] |= abwChoice.getSelectedIndex() << 1;
    }

	if (gptrenCheckBox != null && gptrenCheckBox.isSelected()) {
	  config[CONFIG3L] |= 1 << 5;
	}
	else {
	  config[CONFIG3L] &= (~(1 << 5)) & 0xff;
	}

	if (nott1oscmxCheckBox != null && nott1oscmxCheckBox.isSelected()) {
      config[CONFIG3L] |= 1 << 5;
    }
	else {
	  config[CONFIG3L] &= (~(1 << 5)) & 0xff;
	}

	if (hpolCheckBox != null && hpolCheckBox.isSelected()) {
	  config[CONFIG3L] |= 1 << 4;
	}
	else {
	  config[CONFIG3L] &= (~(1 << 4)) & 0xff;
	}

	if (lpolCheckBox != null && lpolCheckBox.isSelected()) {
	  config[CONFIG3L] |= 1 << 3;
	}
	else {
	  config[CONFIG3L] &= (~(1 << 3)) & 0xff;
	}

	if (pwmpinCheckBox != null && pwmpinCheckBox.isSelected()) {
	  config[CONFIG3L] |= 1 << 2;
	}
	else {
	  config[CONFIG3L] &= (~(1 << 2)) & 0xff;
	}

    if (pmChoice != null) {
      config[CONFIG3L] &= (~7) & 0xff;
      config[CONFIG3L] |= pmChoice.getSelectedIndex();
	}

    if (mclreCheckBox != null && mclreCheckBox.isSelected()) {
      config[CONFIG3H] |= 1 << 7;
    }
    else {
      config[CONFIG3H] &= (~(1 << 7)) & 0xff;
    }

	if (exclkmxCheckBox != null && exclkmxCheckBox.isSelected()) {
	  config[CONFIG3H] |= 1 << 4;
	}
	else {
	  config[CONFIG3H] &= (~(1 << 4)) & 0xff;
	}

	if (pwm4mxCheckBox != null && pwm4mxCheckBox.isSelected()) {
	  config[CONFIG3H] |= 1 << 3;
	}
	else {
	  config[CONFIG3H] &= (~(1 << 3)) & 0xff;
	}

	if (sspmxCheckBox != null && sspmxCheckBox.isSelected()) {
	  config[CONFIG3H] |= 1 << 2;
	}
	else {
	  config[CONFIG3H] &= (~(1 << 2)) & 0xff;
	}

	if (lpt1oscCheckBox != null && lpt1oscCheckBox.isSelected()) {
	  config[CONFIG3H] |= 1 << 2;
	}
	else {
	  config[CONFIG3H] &= (~(1 << 2)) & 0xff;
	}

	if (pbadenCheckBox != null && pbadenCheckBox.isSelected()) {
	  config[CONFIG3H] |= 1 << 1;
	}
	else {
	  config[CONFIG3H] &= (~(1 << 1)) & 0xff;
	}

	if (t1oscmxCheckBox != null && t1oscmxCheckBox.isSelected()) {
	  config[CONFIG3H] |= 1 << 1;
	}
	else {
	  config[CONFIG3H] &= (~(1 << 1)) & 0xff;
	}
	
	if (eccpmxCheckBox != null && eccpmxCheckBox.isSelected()) {
	  config[CONFIG3H] |= 1 << 1;
	}
	else {
	  config[CONFIG3H] &= (~(1 << 1)) & 0xff;
	}

	if (fltamxCheckBox != null && fltamxCheckBox.isSelected()) {
	  config[CONFIG3H] |= 1;
	}
	else {
	  config[CONFIG3H] &= 0xfe;
	}

	if (ccp2mxCheckBox != null && ccp2mxCheckBox.isSelected()) {
	  config[CONFIG3H] |= 1;
	}
	else {
	  config[CONFIG3H] &= 0xfe;
	}

    if (debugCheckBox.isSelected()) {
      config[CONFIG4L] |= 1 << 7;
    }
    else {
      config[CONFIG4L] &= (~(1 << 7)) & 0xff;
    }

    if (xinstCheckBox != null && xinstCheckBox.isSelected()) {
        config[CONFIG4L] |= 1 << 6;
      }
      else {
        config[CONFIG4L] &= (~(1 << 6)) & 0xff;
      }

	if (device.getConfigBits().isBbsiz()) {
      config[CONFIG2H] &= (~0x30) & 0xff;
      config[CONFIG2H] |= bbsizChoice.getSelectedIndex() << 4;
	}
	
    if (icprtCheckBox != null && icprtCheckBox.isSelected()) {
      config[CONFIG4L] |= 1 << 5;
    }
    else {
      config[CONFIG4L] &= (~(1 << 5)) & 0xff;
    }

    if (lvpCheckBox.isSelected()) {
      config[CONFIG4L] |= 1 << 2;
    }
    else {
      config[CONFIG4L] &= (~(1 << 2)) & 0xff;
    }

    if (stvrenCheckBox.isSelected()) {
      config[CONFIG4L] |= 1;
    }
    else {
      config[CONFIG4L] &= 0xfe;
    }
   
    if (cpCheckBox != null && cpCheckBox.isSelected()) {
      config[CONFIG5L] |= 1;
    }
    else {
      config[CONFIG5L] &= 0xfe;
    }

    if (cp0CheckBox != null && cp0CheckBox.isSelected()) {
      config[CONFIG5L] |= 1;
    }
    else {
      config[CONFIG5L] &= 0xfe;
    }

    if (cp1CheckBox != null && cp1CheckBox.isSelected()) {
      config[CONFIG5L] |= 1 << 1;
    }
    else {
      config[CONFIG5L] &= (~(1 << 1)) & 0xff;
    }

    if (cp2CheckBox != null && cp2CheckBox.isSelected()) {
      config[CONFIG5L] |= 1 << 2;
    }
    else {
      config[CONFIG5L] &= (~(1 << 2)) & 0xff;
    }

    if (cp3CheckBox != null && cp3CheckBox.isSelected()) {
      config[CONFIG5L] |= 1 << 3;
    }
    else {
      config[CONFIG5L] &= (~(1 << 3)) & 0xff;
    }

	if (cp4CheckBox != null && cp4CheckBox.isSelected()) {
	  config[CONFIG5L] |= 1 << 4;
	}
	else {
	  config[CONFIG5L] &= (~(1 << 4)) & 0xff;
	}

	if (cp5CheckBox != null && cp5CheckBox.isSelected()) {
	  config[CONFIG5L] |= 1 << 5;
	}
	else {
	  config[CONFIG5L] &= (~(1 << 5)) & 0xff;
	}

	if (cp6CheckBox != null && cp6CheckBox.isSelected()) {
	  config[CONFIG5L] |= 1 << 6;
	}
	else {
	  config[CONFIG5L] &= (~(1 << 6)) & 0xff;
	}

	if (cp7CheckBox != null && cp7CheckBox.isSelected()) {
	  config[CONFIG5L] |= 1 << 7;
	}
	else {
	  config[CONFIG5L] &= (~(1 << 7)) & 0xff;
	}

    if (cpdCheckBox != null && cpdCheckBox.isSelected()) {
      config[CONFIG5H] |= 1 << 7;
    }
    else {
      config[CONFIG5H] &= (~(1 << 7)) & 0xff;
    }

    if (cpbCheckBox != null && cpbCheckBox.isSelected()) {
      config[CONFIG5H] |= 1 << 6;
    }
    else {
      config[CONFIG5H] &= (~(1 << 6)) & 0xff;
    }

    if (wrt0CheckBox != null && wrt0CheckBox.isSelected()) {
      config[CONFIG6L] |= 1;
    }
    else {
      config[CONFIG6L] &= 0xfe;
    }

    if (wrt1CheckBox != null && wrt1CheckBox.isSelected()) {
      config[CONFIG6L] |= 1 << 1;
    }
    else {
      config[CONFIG6L] &= (~(1 << 1)) & 0xff;
    }

    if (wrt2CheckBox != null && wrt2CheckBox.isSelected()) {
      config[CONFIG6L] |= 1 << 2;
    }
    else {
      config[CONFIG6L] &= (~(1 << 2)) & 0xff;
    }

    if (wrt3CheckBox != null && wrt3CheckBox.isSelected()) {
      config[CONFIG6L] |= 1 << 3;
    }
    else {
      config[CONFIG6L] &= (~(1 << 3)) & 0xff;
    }

	if (wrt4CheckBox != null && wrt4CheckBox.isSelected()) {
	  config[CONFIG6L] |= 1 << 4;
	}
	else {
	  config[CONFIG6L] &= (~(1 << 4)) & 0xff;
	}

	if (wrt5CheckBox != null && wrt5CheckBox.isSelected()) {
	  config[CONFIG6L] |= 1 << 5;
	}
	else {
	  config[CONFIG6L] &= (~(1 << 5)) & 0xff;
	}

	if (wrt6CheckBox != null && wrt6CheckBox.isSelected()) {
	  config[CONFIG6L] |= 1 << 6;
	}
	else {
	  config[CONFIG6L] &= (~(1 << 6)) & 0xff;
	}

	if (wrt7CheckBox != null && wrt7CheckBox.isSelected()) {
	  config[CONFIG6L] |= 1 << 7;
	}
	else {
	  config[CONFIG6L] &= (~(1 << 7)) & 0xff;
	}

    if (wrtdCheckBox != null && wrtdCheckBox.isSelected()) {
      config[CONFIG6H] |= 1 << 7;
    }
    else {
      config[CONFIG6H] &= (~(1 << 7)) & 0xff;
    }

    if (wrtbCheckBox != null && wrtbCheckBox.isSelected()) {
      config[CONFIG6H] |= 1 << 6;
    }
    else {
      config[CONFIG6H] &= (~(1 << 6)) & 0xff;
    }

    if (wrtcCheckBox.isSelected()) {
      config[CONFIG6H] |= 1 << 5;
    }
    else {
      config[CONFIG6H] &= (~(1 << 5)) & 0xff;
    }

    if (ebtrCheckBox != null && ebtrCheckBox.isSelected()) {
      config[CONFIG7L] |= 1;
    }
    else {
      config[CONFIG7L] &= 0xfe;
    }

    if (ebtr0CheckBox != null && ebtr0CheckBox.isSelected()) {
      config[CONFIG7L] |= 1;
    }
    else {
      config[CONFIG7L] &= 0xfe;
    }

    if (ebtr1CheckBox != null && ebtr1CheckBox.isSelected()) {
      config[CONFIG7L] |= 1 << 1;
    }
    else {
      config[CONFIG7L] &= (~(1 << 1)) & 0xff;
    }

    if (ebtr2CheckBox != null && ebtr2CheckBox.isSelected()) {
      config[CONFIG7L] |= 1 << 2;
    }
    else {
      config[CONFIG7L] &= (~(1 << 2)) & 0xff;
    }

    if (ebtr3CheckBox != null && ebtr3CheckBox.isSelected()) {
      config[CONFIG7L] |= 1 << 3;
    }
    else {
      config[CONFIG7L] &= (~(1 << 3)) & 0xff;
    }

	if (ebtr4CheckBox != null && ebtr4CheckBox.isSelected()) {
	  config[CONFIG7L] |= 1 << 4;
	}
	else {
	  config[CONFIG7L] &= (~(1 << 4)) & 0xff;
	}

	if (ebtr5CheckBox != null && ebtr5CheckBox.isSelected()) {
	  config[CONFIG7L] |= 1 << 5;
	}
	else {
	  config[CONFIG7L] &= (~(1 << 5)) & 0xff;
	}

	if (ebtr6CheckBox != null && ebtr6CheckBox.isSelected()) {
	  config[CONFIG7L] |= 1 << 6;
	}
	else {
	  config[CONFIG7L] &= (~(1 << 6)) & 0xff;
	}

	if (ebtr7CheckBox != null && ebtr7CheckBox.isSelected()) {
	  config[CONFIG7L] |= 1 << 7;
	}
	else {
	  config[CONFIG7L] &= (~(1 << 7)) & 0xff;
	}

    if (ebtrbCheckBox != null && ebtrbCheckBox.isSelected()) {
      config[CONFIG7H] |= 1 << 6;
    }
    else {
      config[CONFIG7H] &= (~(1 << 6)) & 0xff;
    }
  } // setConfiguration

  class Config1Panel extends JPanel implements ItemListener {
	final static long serialVersionUID = 0;
	
    public Config1Panel() {
      setBorder(BorderFactory.
                createTitledBorder(BorderFactory.createEtchedBorder(), "CONFIG1H"));

	  if (device.getConfigBits().isUsbpll()) {
	    usbpllCheckBox = new JCheckBox("USBPLL");
	    usbpllCheckBox.setToolTipText(BootloaderResourceBundle.getString("USBPLL_TEXT"));
		usbpllCheckBox.addItemListener(this);
		add(usbpllCheckBox);
	  }
	  else
	    usbpllCheckBox = null;
	  
      if (device.getConfigBits().isCpudiv()) {
        JLabel cpudivLabel = new JLabel("CPUDIV:", Label.LEFT);
        add(cpudivLabel);
        cpudivLabel.setToolTipText(BootloaderResourceBundle.getString("CPUDIV_TEXT"));

        initCpudivChoice();
      }
      else
    	cpudivChoice = null;
      
      if (device.getConfigBits().isPlldiv()) {
        JLabel plldivLabel = new JLabel("PLLDIV:", Label.LEFT);
        add(plldivLabel);
        plldivLabel.setToolTipText(BootloaderResourceBundle.getString("PLLDIV_TEXT"));

        plldivChoice = new Choice();
        plldivChoice.addItemListener(this);
		plldivChoice.add("No prescale");
		plldivChoice.add("Divide by 2");
		plldivChoice.add("Divide by 3");
		plldivChoice.add("Divide by 4");
		plldivChoice.add("Divide by 5");
		plldivChoice.add("Divide by 6");
		plldivChoice.add("Divide by 10");
		plldivChoice.add("Divide by 12");
        add(plldivChoice);
      }
      else
      	plldivChoice = null;
     
	  if (device.getConfigBits().isIeso()) {
	    iesoCheckBox = new JCheckBox("IESO");
	    iesoCheckBox.setToolTipText(BootloaderResourceBundle.getString("IESO_TEXT"));
		iesoCheckBox.addItemListener(this);
		add(iesoCheckBox);
	  }
	  else
	    iesoCheckBox = null;
	  
	  if (device.getConfigBits().isFscm()) {
	    fcmenCheckBox = new JCheckBox("FCMEN");
	    fcmenCheckBox.setToolTipText(BootloaderResourceBundle.getString("FCMEN_TEXT"));
		fcmenCheckBox.addItemListener(this);
		add(fcmenCheckBox);
	  }
	  else
	    fcmenCheckBox = null;
      
      if (device.getConfigBits().isOscsen()) {
        oscsenCheckBox = new JCheckBox("!OSCSEN");
        oscsenCheckBox.setToolTipText(BootloaderResourceBundle.getString("OSCSEN_TEXT"));
        oscsenCheckBox.addItemListener(this);
        add(oscsenCheckBox);
	  }
	  else
	  	oscsenCheckBox = null;

      if (device.getConfigBits().isFosc20() || device.getConfigBits().isFosc30()) {
        JLabel foscLabel = new JLabel("FOSC:", Label.LEFT);
        foscLabel.setToolTipText(BootloaderResourceBundle.getString("FOSC_TEXT"));
        add(foscLabel);

        foscChoice = new Choice();
        if (device.configBits.isCpudiv()) {
          foscChoice.add("XT");
          foscChoice.add("XT");
          foscChoice.add("XTPLL");
          foscChoice.add("XTPLL");
          foscChoice.add("ECIO");
          foscChoice.add("EC");
          foscChoice.add("ECPIO");
          foscChoice.add("ECPLL");
          foscChoice.add("INTIO");
          foscChoice.add("INTCKO");
          foscChoice.add("INTXT");
          foscChoice.add("INTHS");
          foscChoice.add("HS");
          foscChoice.add("HS");
          foscChoice.add("HSPLL");
          foscChoice.add("HSPLL");
        }
        else {
          if (!device.name.endsWith("39")) {
            foscChoice.add("LP");
            foscChoice.add("XT");
          }
          else {
		    foscChoice.add("Reserved");
		    foscChoice.add("Reserved");
          }

          foscChoice.add("HS");

          if (device.getConfigBits().isFosc20()) {
            if (device.name.endsWith("2") || device.name.endsWith("8"))
              foscChoice.add("RC");
            else if (device.name.endsWith("39"))
    	      foscChoice.add("Reserved");
            else
      	      foscChoice.add("RC/OSC2 divide-by-4");
          }
          else if (device.name.endsWith("6525") || device.name.endsWith("6621") ||
         		   device.name.endsWith("8525") || device.name.endsWith("8621"))
            foscChoice.add("RC/OSC2 divide-by-4");
          else if (device.name.endsWith("1220") || device.name.endsWith("1320"))
            foscChoice.add("Reserved");
          else
            foscChoice.add("External RC/CLKO RA6");

          if (device.name.endsWith("1220") || device.name.endsWith("1320") ||
          	  device.name.endsWith("2525") || device.name.endsWith("2620") ||
         	  device.name.endsWith("4525") || device.name.endsWith("4620") ||
			  device.name.endsWith("90")) {
            foscChoice.add("EC/CLKO RA6");
            foscChoice.add("EC/port RA6");
          }
          else {
            foscChoice.add("EC/OSC2 divide-by-4");
            foscChoice.add("EC/OSC2 RA6");
          }
         
          foscChoice.add("HS/PLL");

          if (device.getConfigBits().isFosc20()) {
	        if (device.name.endsWith("39"))
		      foscChoice.addItem("Reserved");
		    else if (device.getConfigBits().isFosc20())
		      foscChoice.add("RC/OSC2 RA6");
          }
          else if (device.getConfigBits().isOscsen()) {
            foscChoice.add("RC/OSC2 RA6");
            foscChoice.add("Reserved");
            foscChoice.add("Reserved");
            foscChoice.add("Reserved");
            foscChoice.add("Reserved");
            foscChoice.add("EC/OSC2 RA6 HW PLL");
            foscChoice.add("EC/OSC2 RA6 SW PLL");
            foscChoice.add("HS/SW PLL");
            foscChoice.add("RC/OSC2 RA6");
          }
          else {
        	foscChoice.add("External RC/port RA6");
		    foscChoice.add("Internal oscillator block/port RA6/RA7");
		    foscChoice.add("Internal oscillator block/CLKO RA6");
		    foscChoice.add("External RC/CLKO RA6");
		    foscChoice.add("External RC/CLKO RA6");
		    foscChoice.add("External RC/CLKO RA6");
		    foscChoice.add("External RC/CLKO RA6");
		    foscChoice.add("External RC/CLKO RA6");
		    foscChoice.add("External RC/CLKO RA6");
	      }
        }
        foscChoice.addItemListener(this);
        add(foscChoice);
      }
      else
      	foscChoice = null;
    } // Config1Panel

    private void initCpudivChoice() {
      if (cpudivChoice == null) {
      	cpudivChoice = new Choice();
      
        cpudivChoice.addItemListener(this);

        add(cpudivChoice);
      }
     
      cpudivChoice.removeAll();
      
      int fosc;
      if (foscChoice == null)
      	fosc = config[CONFIG1H] & 0xf;
      else
        fosc = foscChoice.getSelectedIndex();

      if ((fosc & 0xe) == 0xe || fosc == 0x7 || fosc == 0x6 ||
      	  (fosc & 0xe) == 0x2) { // For HSPLL, ECPLL, ECPIO and XTPLL
		  cpudivChoice.add("Primary oscillator divided by 2");
		  cpudivChoice.add("Primary oscillator divided by 3");
		  cpudivChoice.add("Primary oscillator divided by 4");
		  cpudivChoice.add("Primary oscillator divided by 6");
      }
      else {
        cpudivChoice.add("No Postscaler");
        cpudivChoice.add("Primary oscillator divided by 2");
  	    cpudivChoice.add("Primary oscillator divided by 3");
  	    cpudivChoice.add("Primary oscillator divided by 4");
      }
    } // initCpudivChoice
    
    private void selectConfig() {
      int conf = bootloader.getConfig(device.loConfigMem +
                                      CONFIG1L) & 0xff;

      if (usbpllCheckBox != null) {
		if ((conf & (1 << 5)) > 0)
		  usbpllCheckBox.setSelected(true);
		else
		  usbpllCheckBox.setSelected(false);
      }

	  if (plldivChoice != null) {
        int sel;
        sel = (conf & 0x18) >> 3;
        plldivChoice.select(sel);
	  }

	  if (plldivChoice != null) {
        int sel;
        sel = conf & 7;
        plldivChoice.select(sel);
	  }

      conf = bootloader.getConfig(device.loConfigMem + CONFIG1H) & 0xff;

      if (iesoCheckBox != null) {
		if ((conf & (1 << 7)) > 0)
		  iesoCheckBox.setSelected(true);
		else
		  iesoCheckBox.setSelected(false);
      }
      
	  if (fcmenCheckBox != null) {
		if ((conf & (1 << 6)) > 0)
	      fcmenCheckBox.setSelected(true);
		else
		  fcmenCheckBox.setSelected(false);
	  }

	  if (oscsenCheckBox != null) {
        if ((conf & (1 << 5)) > 0)
          oscsenCheckBox.setSelected(true);
        else
          oscsenCheckBox.setSelected(false);
	  }
	   
	  if (foscChoice != null) {
        int sel;
        if (device.getConfigBits().isFosc30())
        	sel = conf & 0xf;
        else
        	sel = conf & 7;
        foscChoice.select(sel);
	  }
    } // selectConfig

    /**
    Event handler for changes in the current selection of the Choices.
    */
    public void itemStateChanged(ItemEvent e) {
      if (e.getItemSelectable() == foscChoice && cpudivChoice != null) {
      	initCpudivChoice();
      }

      // Set the parameters in the panel.
      setConfiguration();
    } // itemStateChanged
  } // Config1Panel

  class Config2Panel extends JPanel implements ItemListener {
	final static long serialVersionUID = 0;
	
    public Config2Panel() {
      setBorder(BorderFactory.
                createTitledBorder(BorderFactory.createEtchedBorder(), "CONFIG2H & CONFIG2L"));

      if (device.getConfigBits().isVregen()) {
        vregenCheckBox = new JCheckBox("VREGEN");
        vregenCheckBox.addItemListener(this);
        vregenCheckBox.setToolTipText(BootloaderResourceBundle.getString("VREGEN_TEXT"));
        add(vregenCheckBox);
      }
      
      JLabel borvLabel = new JLabel("BORV:", Label.LEFT);
      add(borvLabel);
      borvLabel.setToolTipText(BootloaderResourceBundle.getString("BORV_TEXT"));

      borvChoice = new Choice();
	  if (device.name.endsWith("10") || device.name.endsWith("50") ||
	      device.name.endsWith("55") || device.name.endsWith("80") ||
	      device.name.endsWith("85") || device.name.endsWith("90")) {
        borvChoice.add("4.6V");
        borvChoice.add("4.3V");
        borvChoice.add("2.8V");
        borvChoice.add("2.1V");
	  }
	  else if (device.name.endsWith("2525") || device.name.endsWith("2620") ||
	  		   device.name.endsWith("4525") || device.name.endsWith("4620") ||
	  		   device.name.endsWith("2423") || device.name.endsWith("2523") ||
	  		   device.name.endsWith("4423") || device.name.endsWith("4523") ||
	  		   device.name.endsWith("6722") || device.name.endsWith("8722") ||
	  		   device.name.endsWith("6527") || device.name.endsWith("6622") ||
	  		   device.name.endsWith("6627") || device.name.endsWith("8527") ||
	  		   device.name.endsWith("8622") || device.name.endsWith("8627")) {
        borvChoice.add("Maximum setting");
        borvChoice.add("Intermediate setting >");
        borvChoice.add("Intermediate setting <");
        borvChoice.add("Minimum setting");
	  }
	  else {
        borvChoice.add("4.5V");
        borvChoice.add("4.2V");
        borvChoice.add("2.7V");
	    if (device.name.equals("PIC18F6620") || device.name.equals("PIC18F6720") ||
		    device.name.equals("PIC18F8620") || device.name.equals("PIC18F8720") ||
		    device.name.endsWith("2") || device.name.endsWith("39"))
		  borvChoice.add("2.5V");
        else if (device.name.endsWith("1220") || device.name.endsWith("1320") ||
        		 device.name.endsWith("31"))
  		  borvChoice.add("Reserved");
        else
		  borvChoice.add("2.0V");
	  }
      borvChoice.addItemListener(this);
      add(borvChoice);

      
      if (device.getConfigBits().isBoren()) {
        bodenCheckBox = new JCheckBox("BODEN");
        bodenCheckBox.addItemListener(this);
        bodenCheckBox.setToolTipText(BootloaderResourceBundle.getString("BODEN_TEXT"));
        add(bodenCheckBox);
      }
      
      if (device.getConfigBits().isBoren10()) {
        JLabel borenLabel = new JLabel("BOREN:", Label.LEFT);
        add(borenLabel);
        borenLabel.setToolTipText(BootloaderResourceBundle.getString("BOREN_TEXT"));

        borenChoice = new Choice();
        borenChoice.addItemListener(this);
		borenChoice.add("BOR HW");
		borenChoice.add("BOR HW/disabled sleep mode");
		borenChoice.add("BOR SW");
		borenChoice.add("BOR disabled HW/SW");
        add(borenChoice);
      }
      else
      	borenChoice = null;
      
      pwrtenCheckBox = new JCheckBox("!PWRTEN");
      pwrtenCheckBox.setToolTipText(BootloaderResourceBundle.getString("PWRT_TEXT"));
      pwrtenCheckBox.addItemListener(this);
      add(pwrtenCheckBox);

	  if (device.getConfigBits().isWinen()) {
		winenCheckBox = new JCheckBox("!WINEN");
		winenCheckBox.setToolTipText(BootloaderResourceBundle.getString("WINEN_TEXT"));
		winenCheckBox.addItemListener(this);
		add(winenCheckBox);
	  }
	  else
		winenCheckBox = null;
      
      JLabel wdtpLabel = new JLabel("WDTP:", Label.LEFT);
      wdtpLabel.setToolTipText(BootloaderResourceBundle.getString("WDTP_TEXT"));
      add(wdtpLabel);

      wdtpChoice = new Choice();
      wdtpChoice.add("1:1");
      wdtpChoice.add("1:2");
      wdtpChoice.add("1:4");
      wdtpChoice.add("1:8");
      wdtpChoice.add("1:16");
      wdtpChoice.add("1:32");
      wdtpChoice.add("1:64");
      wdtpChoice.add("1:128");
      if (device.getConfigBits().isWdtps30()) {
        wdtpChoice.add("1:256");
        wdtpChoice.add("1:512");
        wdtpChoice.add("1:1,024");
        wdtpChoice.add("1:2,048");
        wdtpChoice.add("1:4,096");
        wdtpChoice.add("1:8,192");
        wdtpChoice.add("1:16,384");
        wdtpChoice.add("1:32,768");   	
      }
      wdtpChoice.addItemListener(this);
      add(wdtpChoice);

      wdtenCheckBox = new JCheckBox("WDTEN");
      wdtenCheckBox.setToolTipText(BootloaderResourceBundle.getString("WDTEN_TEXT"));
      wdtenCheckBox.addItemListener(this);
      add(wdtenCheckBox);
    } // Config2Panel

    private void selectConfig() {
      int conf = bootloader.getConfig(device.loConfigMem +
                                      CONFIG2L) & 0xff;

      int sel;
      
      if (vregenCheckBox != null) {
        if ((conf & (1 << 5)) > 0)
          vregenCheckBox.setSelected(true);
        else
          vregenCheckBox.setSelected(false);
      }
      
      if (device.getConfigBits().isBoren10()) {
        sel = (conf & (3 << 3)) >> 3;
        borvChoice.select(sel);
        sel = (conf & (3 << 1)) >> 1;
        borenChoice.select(sel);
      }
      else {
        sel = (conf & (3 << 2)) >> 2;
        borvChoice.select(sel);
        if ((conf & (1 << 1)) > 0)
          bodenCheckBox.setSelected(true);
        else
          bodenCheckBox.setSelected(false);
      }
      
      if ((conf & 1) > 0)
        pwrtenCheckBox.setSelected(true);
      else
        pwrtenCheckBox.setSelected(false);

      conf = bootloader.getConfig(device.loConfigMem +
                                  CONFIG2H) & 0xff;

	  if (winenCheckBox != null) {
		if ((conf & 0x20) > 0)
		  winenCheckBox.setSelected(true);
		 else
		   winenCheckBox.setSelected(false);
	  }

	  if (device.getConfigBits().isWdtps30())
        sel = (conf & (0xf << 1)) >> 1;
	  else
	  	sel = (conf & (7 << 1)) >> 1;
	  wdtpChoice.select(sel);

      if ((conf & 1) > 0)
        wdtenCheckBox.setSelected(true);
      else
        wdtenCheckBox.setSelected(false);
    } // selectConfig

    /**
    Event handler for changes in the current selection of the Choices.
    */
    public void itemStateChanged(ItemEvent e) {
      // Set the parameters in the panel.
      setConfiguration();
    } // itemStateChanged
  } // Config2Panel

  class Config3LPanel extends JPanel implements ItemListener {
	final static long serialVersionUID = 0;

	public Config3LPanel() {
	  setBorder(BorderFactory.
				createTitledBorder(BorderFactory.createEtchedBorder(), "CONFIG3L"));

	  if (device.getConfigBits().isWait()) {
	    waitCheckBox = new JCheckBox("WAIT");
	    waitCheckBox.setToolTipText(BootloaderResourceBundle.getString("WAIT_TEXT"));
	    waitCheckBox.addItemListener(this);
	    add(waitCheckBox);
	  }
	  else
	  	waitCheckBox = null;
	  
	  if (device.getConfigBits().isBw()) {
	    bwCheckBox = new JCheckBox("!BW");
	    bwCheckBox.setToolTipText(BootloaderResourceBundle.getString("BW_TEXT"));
	    bwCheckBox.addItemListener(this);
	    add(bwCheckBox);
	  }
	  else
	  	bwCheckBox = null;
	  
      if (device.getConfigBits().isAbw()) {
        JLabel abwLabel = new JLabel("ABW:", Label.LEFT);
        add(abwLabel);
        abwLabel.setToolTipText(BootloaderResourceBundle.getString("ABW_TEXT"));

        abwChoice = new Choice();
        abwChoice.addItemListener(this);
  		abwChoice.add("8-bit address bus");
  		abwChoice.add("12-bit address bus");
  		abwChoice.add("16-bit address bus");
  		abwChoice.add("20-bit address bus");
        add(abwChoice);
      }
      else
        abwChoice = null;
      
	  if (device.getConfigBits().isGptren()) {
	    gptrenCheckBox = new JCheckBox("!GPTREN");
	    gptrenCheckBox.setToolTipText(BootloaderResourceBundle.getString("GPTREN_TEXT"));
	    gptrenCheckBox.addItemListener(this);
	    add(gptrenCheckBox);
	  }
	  else
	  	gptrenCheckBox = null;
	  
	  if (device.getConfigBits().isNott1oscmx()) {
	    nott1oscmxCheckBox = new JCheckBox("!T1OSCMX");
	    nott1oscmxCheckBox.setToolTipText(BootloaderResourceBundle.getString("NOTT1OSCMX_TEXT"));
	    nott1oscmxCheckBox.addItemListener(this);
	    add(nott1oscmxCheckBox);
	  }
	  else
	  	nott1oscmxCheckBox = null;
	  
	  if (device.getConfigBits().isHpol()) {
	    hpolCheckBox = new JCheckBox("HPOL");
	    hpolCheckBox.setToolTipText(BootloaderResourceBundle.getString("HPOL_TEXT"));
	    hpolCheckBox.addItemListener(this);
	    add(hpolCheckBox);
	  }
	  else
	  	hpolCheckBox = null;
	  
	  if (device.getConfigBits().isLpol()) {
	    lpolCheckBox = new JCheckBox("LPOL");
	    lpolCheckBox.setToolTipText(BootloaderResourceBundle.getString("LPOL_TEXT"));
	    lpolCheckBox.addItemListener(this);
	    add(lpolCheckBox);
	  }
	  else
	  	lpolCheckBox = null;
	  
	  if (device.getConfigBits().isLpol()) {
	    pwmpinCheckBox = new JCheckBox("PWMPIN");
	    pwmpinCheckBox.setToolTipText(BootloaderResourceBundle.getString("PWMPIN_TEXT"));
	    pwmpinCheckBox.addItemListener(this);
	    add(pwmpinCheckBox);
	  }
	  else
	  	pwmpinCheckBox = null;
	  
	  JLabel pmLabel = new JLabel("PM:", Label.LEFT);
      add(pmLabel);

	  if (device.getConfigBits().isPm()) {
	    pmChoice = new Choice();
	    pmChoice.addItem("Extended Microcontroller Mode");
	    pmChoice.addItem("Microprocessor with Boot Block Mode");
	    pmChoice.addItem("Microprocessor Mode");
	    pmChoice.addItem("Microcontroller Mode");
	    add(pmChoice);
	  }
	  else
	  	pmChoice = null;
	} // Config3LPanel
	
	private void selectConfig() {
	  int conf = bootloader.getConfig(device.loConfigMem +
									  CONFIG3L) & 0xff;

	  if (waitCheckBox != null) {
		if ((conf & 0x80) > 0)
		  waitCheckBox.setSelected(true);
		else
		  waitCheckBox.setSelected(false);
	  }
      
	  if (bwCheckBox != null) {
		if ((conf & 0x40) > 0)
		  bwCheckBox.setSelected(true);
		else
		  bwCheckBox.setSelected(false);
	  }
      
	  if (gptrenCheckBox != null) {
		if ((conf & 0x20) > 0)
		  gptrenCheckBox.setSelected(true);
		else
		  gptrenCheckBox.setSelected(false);
	  }
 
	  if (nott1oscmxCheckBox != null) {
		if ((conf & 0x20) > 0)
		  nott1oscmxCheckBox.setSelected(true);
		else
		  nott1oscmxCheckBox.setSelected(false);
	  }
      
	  if (hpolCheckBox != null) {
		if ((conf & 0x10) > 0)
		  hpolCheckBox.setSelected(true);
		else
		  hpolCheckBox.setSelected(false);
	  }

	  if (lpolCheckBox != null) {
		if ((conf & 0x08) > 0)
		  lpolCheckBox.setSelected(true);
		else
		  lpolCheckBox.setSelected(false);
	  }

	  if (pwmpinCheckBox != null) {
		if ((conf & 0x04) > 0)
		  pwmpinCheckBox.setSelected(true);
		else
		  pwmpinCheckBox.setSelected(false);
	  }
	  
	  if (pmChoice != null) {
	    int sel = conf & 3;
	    pmChoice.select(sel);
	  }
	} // selectConfig

	/**
	Event handler for changes in the current selection of the Choices.
	*/
	public void itemStateChanged(ItemEvent e) {
	  // Set the parameters in the panel.
	  setConfiguration();
	} // itemStateChanged
  } // Config3LPanel
  
  class Config3HPanel extends JPanel implements ItemListener {
	final static long serialVersionUID = 0;

    public Config3HPanel() {
      setBorder(BorderFactory.
                createTitledBorder(BorderFactory.createEtchedBorder(), "CONFIG3H"));

      if (device.getConfigBits().isMclre()) {
	    mclreCheckBox = new JCheckBox("MCLRE");
	    mclreCheckBox.setToolTipText(BootloaderResourceBundle.getString("MCLRE_TEXT"));
	    mclreCheckBox.addItemListener(this);
	    add(mclreCheckBox);
      }
      else
        mclreCheckBox = null;
        
	  if (device.getConfigBits().isExclkmx()) {
		exclkmxCheckBox = new JCheckBox("EXCLKMX");
		exclkmxCheckBox.setToolTipText(BootloaderResourceBundle.getString("EXCLKMX_TEXT"));
		exclkmxCheckBox.addItemListener(this);
		add(exclkmxCheckBox);
	  }
	  else
	  	exclkmxCheckBox = null;
	  
	  if (device.getConfigBits().isPwm4mx()) {
		pwm4mxCheckBox = new JCheckBox("PWM4MX");
		pwm4mxCheckBox.setToolTipText(BootloaderResourceBundle.getString("PWM4MX_TEXT"));
		pwm4mxCheckBox.addItemListener(this);
		add(pwm4mxCheckBox);
	  }
	  else
	  	pwm4mxCheckBox = null;
	  
	  if (device.getConfigBits().isSspmx()) {
		sspmxCheckBox = new JCheckBox("SSPMX");
		sspmxCheckBox.setToolTipText(BootloaderResourceBundle.getString("SSPMX_TEXT"));
		sspmxCheckBox.addItemListener(this);
		add(sspmxCheckBox);
	  }
	  else
	  	sspmxCheckBox = null;
	  
      if (device.getConfigBits().isFltamx()) {
		fltamxCheckBox = new JCheckBox("FLTAMX");
		fltamxCheckBox.setToolTipText(BootloaderResourceBundle.getString("FLTAMX_TEXT"));
		fltamxCheckBox.addItemListener(this);
		add(fltamxCheckBox);
	  }
	  else
		fltamxCheckBox = null;
            
	  if (device.getConfigBits().isLpt1osc()) {
		lpt1oscCheckBox = new JCheckBox("LPT1OSC");
		lpt1oscCheckBox.setToolTipText(BootloaderResourceBundle.getString("LPT1OSC_TEXT"));
		lpt1oscCheckBox.addItemListener(this);
		add(lpt1oscCheckBox);
	  }
	  else
	  	lpt1oscCheckBox = null;
      
	  if (device.getConfigBits().isPbaden()) {
		pbadenCheckBox = new JCheckBox("PBADEN");
		pbadenCheckBox.setToolTipText(BootloaderResourceBundle.getString("PBADEN_TEXT"));
		pbadenCheckBox.addItemListener(this);
		add(pbadenCheckBox);
	  }
	  else
	    pbadenCheckBox = null;
 
	  if (device.getConfigBits().isT1oscmx()) {
		t1oscmxCheckBox = new JCheckBox("T1OSCMX");
		t1oscmxCheckBox.setToolTipText(BootloaderResourceBundle.getString("T1OSCMX_TEXT"));
		t1oscmxCheckBox.addItemListener(this);
		add(t1oscmxCheckBox);
	  }
	  else
	    t1oscmxCheckBox = null;
      
	  if (device.getConfigBits().isEccpmx()) {
		eccpmxCheckBox = new JCheckBox("ECCPMX");
		eccpmxCheckBox.setToolTipText(BootloaderResourceBundle.getString("ECCPMX_TEXT"));
		eccpmxCheckBox.addItemListener(this);
		add(eccpmxCheckBox);
	  }
	  else
	    eccpmxCheckBox = null;
      
	  if (device.getConfigBits().isCcp2mx()) {
        ccp2mxCheckBox = new JCheckBox("CCP2MX");
        ccp2mxCheckBox.setToolTipText(BootloaderResourceBundle.getString("CCP2MX_TEXT"));
        ccp2mxCheckBox.addItemListener(this);
        add(ccp2mxCheckBox);
	  }
	  else
	    ccp2mxCheckBox = null;
    } // Config3HPanel

    private void selectConfig() {
      int conf = bootloader.getConfig(device.loConfigMem +
                                      CONFIG3H) & 0xff;

      if (mclreCheckBox != null) {
	    if ((conf & 0x80) > 0)
		  mclreCheckBox.setSelected(true);
	    else
		  mclreCheckBox.setSelected(false);
      }
      
	  if (exclkmxCheckBox != null) {
		if ((conf & 0x10) > 0)
		  exclkmxCheckBox.setSelected(true);
		else
		  exclkmxCheckBox.setSelected(false);
	  }

	  if (pwm4mxCheckBox != null) {
		if ((conf & 0x08) > 0)
		  exclkmxCheckBox.setSelected(true);
		else
		  exclkmxCheckBox.setSelected(false);
	  }

	  if (sspmxCheckBox != null) {
		if ((conf & 0x04) > 0)
		  sspmxCheckBox.setSelected(true);
		else
		  sspmxCheckBox.setSelected(false);
	  }

      if (pbadenCheckBox != null) {
	    if ((conf & 0x02) > 0)
	  	  pbadenCheckBox.setSelected(true);
	    else
		  pbadenCheckBox.setSelected(false);
      }

      if (t1oscmxCheckBox != null) {
	    if ((conf & 0x02) > 0)
	  	  t1oscmxCheckBox.setSelected(true);
	    else
		  t1oscmxCheckBox.setSelected(false);
      }

      if (eccpmxCheckBox != null) {
	    if ((conf & 0x02) > 0)
	  	  eccpmxCheckBox.setSelected(true);
	    else
		  eccpmxCheckBox.setSelected(false);
      }
    
	  if (fltamxCheckBox != null) {		
		if ((conf & 1) > 0)
		  fltamxCheckBox.setSelected(true);
		else
		  fltamxCheckBox.setSelected(false);
	  }

	  if (ccp2mxCheckBox != null) {		
		if ((conf & 1) > 0)
		  ccp2mxCheckBox.setSelected(true);
		else
		  ccp2mxCheckBox.setSelected(false);
	  }
    } // selectConfig

    /**
    Event handler for changes in the current selection of the Choices.
    */
    public void itemStateChanged(ItemEvent e) {
      // Set the parameters in the panel.
      setConfiguration();
    } // itemStateChanged
  } // Config3HPanel

  class Config4LPanel extends JPanel implements ItemListener {
	final static long serialVersionUID = 0;

    public Config4LPanel() {
      setBorder(BorderFactory.
                createTitledBorder(BorderFactory.createEtchedBorder(), "CONFIG4L"));

      debugCheckBox = new JCheckBox("!DEBUG");
      debugCheckBox.setToolTipText(BootloaderResourceBundle.getString("DEBUG_TEXT"));
      debugCheckBox.addItemListener(this);
      add(debugCheckBox);

      if (device.getConfigBits().isXinst()) {
        xinstCheckBox = new JCheckBox("XINST");
        xinstCheckBox.setToolTipText(BootloaderResourceBundle.getString("XINST_TEXT"));
        xinstCheckBox.addItemListener(this);
        add(xinstCheckBox);
      }
      else
      	xinstCheckBox = null;
      
      if (device.getConfigBits().isBbsiz()) {
        JLabel bbsizLabel = new JLabel("BBSIZ:", Label.LEFT);
        add(bbsizLabel);
        bbsizLabel.setToolTipText(BootloaderResourceBundle.getString("BBSIZ_TEXT"));

        bbsizChoice = new Choice();
        bbsizChoice.addItemListener(this);
		bbsizChoice.add("8K word");
		bbsizChoice.add("4K word");
		bbsizChoice.add("2K word");
		bbsizChoice.add("1K word");
        add(bbsizChoice);
      }
      else
      	bbsizChoice = null;
     
      if (device.getConfigBits().isIcprt()) {
        icprtCheckBox = new JCheckBox("ICPRT");
        icprtCheckBox.setToolTipText(BootloaderResourceBundle.getString("ICPRT_TEXT"));
        icprtCheckBox.addItemListener(this);
        add(icprtCheckBox);
      }
      else
      	icprtCheckBox = null;
      
      lvpCheckBox = new JCheckBox("LVP");
      lvpCheckBox.setToolTipText(BootloaderResourceBundle.getString("LVP_TEXT"));
      lvpCheckBox.addItemListener(this);
      add(lvpCheckBox);

      stvrenCheckBox = new JCheckBox("STVREN");
      stvrenCheckBox.setToolTipText(BootloaderResourceBundle.getString("STVREN_TEXT"));
      stvrenCheckBox.addItemListener(this);
      add(stvrenCheckBox);
    } // Config4LPanel

    private void selectConfig() {
      int conf = bootloader.getConfig(device.loConfigMem +
                                      CONFIG4L) & 0xff;

      if ((conf & (1 << 7)) > 0)
        debugCheckBox.setSelected(true);
      else
        debugCheckBox.setSelected(false);
 
      if (xinstCheckBox != null) {
        if ((conf & (1 << 6)) > 0)
          xinstCheckBox.setSelected(true);
        else
          xinstCheckBox.setSelected(false);
      }

	  if (bbsizChoice != null) {
        int sel = (conf & (3 << 4)) >> 4;
	    bbsizChoice.select(sel);
	  }
	  
      if (icprtCheckBox != null) {
        if ((conf & (1 << 5)) > 0)
          icprtCheckBox.setSelected(true);
        else
          icprtCheckBox.setSelected(false);
      }
      
      if ((conf & (1 << 2)) > 0)
        lvpCheckBox.setSelected(true);
      else
        lvpCheckBox.setSelected(false);

      if ((conf & 1) > 0)
        stvrenCheckBox.setSelected(true);
      else
        stvrenCheckBox.setSelected(false);
   } // selectConfig

    /**
    Event handler for changes in the current selection of the Choices.
    */
    public void itemStateChanged(ItemEvent e) {
      // Set the parameters in the panel.
      setConfiguration();
    } // itemStateChanged
  } // Config4LPanel

  class Config5Panel extends JPanel implements ItemListener {
	final static long serialVersionUID = 0;

    public Config5Panel() {
      setBorder(BorderFactory.
                createTitledBorder(BorderFactory.createEtchedBorder(),
                "CONFIG5L & CONFIG5H"));

      if (device.getConfigBits().isCp7()) {
        cp7CheckBox = new JCheckBox("CP7");
        cp7CheckBox.setToolTipText(BootloaderResourceBundle.getString("CP7_TEXT"));
        cp7CheckBox.addItemListener(this);
        add(cp7CheckBox);
      }
      else
      	cp7CheckBox = null;
      
      if (device.getConfigBits().isCp6()) {
        cp6CheckBox = new JCheckBox("CP6");
        cp6CheckBox.setToolTipText(BootloaderResourceBundle.getString("CP6_TEXT"));
        cp6CheckBox.addItemListener(this);
        add(cp6CheckBox);
      }
      else
        cp6CheckBox = null;

      if (device.getConfigBits().isCp5()) {
        cp5CheckBox = new JCheckBox("CP5");
        cp5CheckBox.setToolTipText(BootloaderResourceBundle.getString("CP5_TEXT"));
        cp5CheckBox.addItemListener(this);
        add(cp5CheckBox);
      }
      else
        cp5CheckBox = null;

      if (device.getConfigBits().isCp4()) {
        cp4CheckBox = new JCheckBox("CP4");
        cp4CheckBox.setToolTipText(BootloaderResourceBundle.getString("CP4_TEXT"));
        cp4CheckBox.addItemListener(this);
        add(cp4CheckBox); 
      }
      else
        cp4CheckBox = null;

	  if (device.getConfigBits().isCp3()) {
        cp3CheckBox = new JCheckBox("CP3");
        cp3CheckBox.setToolTipText(BootloaderResourceBundle.getString("CP3_TEXT"));
        cp3CheckBox.addItemListener(this);
        add(cp3CheckBox);
	  }
      else
        cp3CheckBox = null;
        
	  if (device.getConfigBits().isCp2()) {
        cp2CheckBox = new JCheckBox("CP2");
        cp2CheckBox.setToolTipText(BootloaderResourceBundle.getString("CP2_TEXT"));
        cp2CheckBox.addItemListener(this);
        add(cp2CheckBox);
	  }
	  else
	    cp2CheckBox = null;
	    
	  if (device.getConfigBits().isCp1()) {
        cp1CheckBox = new JCheckBox("CP1");
        cp1CheckBox.setToolTipText(BootloaderResourceBundle.getString("CP1_TEXT"));
        cp1CheckBox.addItemListener(this);
        add(cp1CheckBox);
	  }
	  else
	    cp1CheckBox = null;
	  	
	  if (device.getConfigBits().isCp0()) {
        cp0CheckBox = new JCheckBox("CP0");
        cp0CheckBox.setToolTipText(BootloaderResourceBundle.getString("CP0_TEXT"));
        cp0CheckBox.addItemListener(this);
        add(cp0CheckBox);
	  }
	  else
	    cp0CheckBox = null;
	  
	  if (device.getConfigBits().isCp()) {
        cpCheckBox = new JCheckBox("CP");
        cpCheckBox.setToolTipText(BootloaderResourceBundle.getString("CP_TEXT"));
        cpCheckBox.addItemListener(this);
        add(cpCheckBox);
	  }
	  else
	    cpCheckBox = null;
	  
	  if (device.getConfigBits().isCpd()) {
        cpdCheckBox = new JCheckBox("CPD");
        cpdCheckBox.setToolTipText(BootloaderResourceBundle.getString("CPD_TEXT"));
        cpdCheckBox.addItemListener(this);
        add(cpdCheckBox);
	  }
	  else
	    cpdCheckBox = null;
	  
	  if (device.getConfigBits().isCpb()) {
        cpbCheckBox = new JCheckBox("CPB");
        cpbCheckBox.setToolTipText(BootloaderResourceBundle.getString("CPB_TEXT"));
        cpbCheckBox.addItemListener(this);
        add(cpbCheckBox);
	  }
	  else
	    cpbCheckBox = null;
    } // Config5Panel

    private void selectConfig() {
      int conf = bootloader.getConfig(device.loConfigMem +
                                      CONFIG5L) & 0xff;

      if (cp7CheckBox != null) {
        if ((conf & (1 << 7)) > 0)
          cp7CheckBox.setSelected(true);
        else
          cp7CheckBox.setSelected(false);
      }
      
      if (cp6CheckBox != null) {
        if ((conf & (1 << 6)) > 0)
          cp6CheckBox.setSelected(true);
        else
          cp6CheckBox.setSelected(false);
      }
        
      if (cp5CheckBox != null) {
        if ((conf & (1 << 5)) > 0)
          cp5CheckBox.setSelected(true);
        else
          cp5CheckBox.setSelected(false);
      }
      
      if (cp4CheckBox != null) {
        if ((conf & (1 << 4)) > 0)
          cp4CheckBox.setSelected(true);
        else
          cp4CheckBox.setSelected(false);
      }

      if (cp3CheckBox != null) {
        if ((conf & (1 << 3)) > 0)
          cp3CheckBox.setSelected(true);
        else
          cp3CheckBox.setSelected(false);
      }
      
      if (cp2CheckBox != null) {
      	if ((conf & (1 << 2)) > 0)
          cp2CheckBox.setSelected(true);
        else
          cp2CheckBox.setSelected(false);
      }
      
      if (cp1CheckBox != null) {
        if ((conf & (1 << 1)) > 0)
          cp1CheckBox.setSelected(true);
        else
          cp1CheckBox.setSelected(false);
      }
      
      if (cp0CheckBox != null) {
        if ((conf & 1) > 0)
          cp0CheckBox.setSelected(true);
        else
          cp0CheckBox.setSelected(false);
      }
      
      if (cpCheckBox != null) {
        if ((conf & 1) > 0)
          cpCheckBox.setSelected(true);
        else
          cpCheckBox.setSelected(false);
      }
      
      conf = bootloader.getConfig(device.loConfigMem + CONFIG5H) & 0xff;

      if (cpdCheckBox != null) {
        if ((conf & (1 << 7)) > 0)
          cpdCheckBox.setSelected(true);
        else
          cpdCheckBox.setSelected(false);
      }
      
      if (cpbCheckBox != null) {
        if ((conf & (1 << 6)) > 0)
          cpbCheckBox.setSelected(true);
        else
          cpbCheckBox.setSelected(false);
      }
   } // selectConfig

    /**
    Event handler for changes in the current selection of the Choices.
    */
    public void itemStateChanged(ItemEvent e) {
      // Set the parameters in the panel.
      setConfiguration();
    } // itemStateChanged
  } // Config5Panel

  class Config6Panel extends JPanel implements ItemListener {
	final static long serialVersionUID = 0;

    public Config6Panel() {
      setBorder(BorderFactory.
                createTitledBorder(BorderFactory.createEtchedBorder(),
                "CONFIG6L & CONFIG6H"));

      if (device.getConfigBits().isWrt7()) {
        wrt7CheckBox = new JCheckBox("WRT7");
        wrt7CheckBox.setToolTipText(BootloaderResourceBundle.getString("WRT7_TEXT"));
        wrt7CheckBox.addItemListener(this);
        add(wrt7CheckBox);
      }
      else
      	wrt7CheckBox = null;
      
      if (device.getConfigBits().isWrt6()) {
        wrt6CheckBox = new JCheckBox("WRT6");
        wrt6CheckBox.setToolTipText(BootloaderResourceBundle.getString("WRT6_TEXT"));
        wrt6CheckBox.addItemListener(this);
        add(wrt6CheckBox);
      }
      else
      	wrt6CheckBox = null;
      
      if (device.getConfigBits().isWrt5()) {
        wrt5CheckBox = new JCheckBox("WRT5");
        wrt5CheckBox.setToolTipText(BootloaderResourceBundle.getString("WRT5_TEXT"));
        wrt5CheckBox.addItemListener(this);
        add(wrt5CheckBox);
      }
      else
    	wrt5CheckBox = null;

      if (device.getConfigBits().isWrt4()) {
        wrt4CheckBox = new JCheckBox("WRT4");
        wrt4CheckBox.setToolTipText(BootloaderResourceBundle.getString("WRT4_TEXT"));
        wrt4CheckBox.addItemListener(this);
        add(wrt4CheckBox);
      }
	  else {
		wrt4CheckBox = null;
	  }

      if (device.getConfigBits().isWrt3()) {
        wrt3CheckBox = new JCheckBox("WRT3");
        wrt3CheckBox.setToolTipText(BootloaderResourceBundle.getString("WRT3_TEXT"));
        wrt3CheckBox.addItemListener(this);
        add(wrt3CheckBox);
	  }
	  else
		wrt3CheckBox = null;

	  if (device.getConfigBits().isWrt2()) {
        wrt2CheckBox = new JCheckBox("WRT2");
        wrt2CheckBox.setToolTipText(BootloaderResourceBundle.getString("WRT2_TEXT"));
        wrt2CheckBox.addItemListener(this);
        add(wrt2CheckBox);
	  }
	  else
	    wrt2CheckBox = null;
	    
	  if (device.getConfigBits().isWrt1()) {
      wrt1CheckBox = new JCheckBox("WRT1");
      wrt1CheckBox.setToolTipText(BootloaderResourceBundle.getString("WRT1_TEXT"));
      wrt1CheckBox.addItemListener(this);
      add(wrt1CheckBox);
	  }
	  else
	    wrt1CheckBox = null;

	  if (device.getConfigBits().isWrt0()) {
        wrt0CheckBox = new JCheckBox("WRT0");
        wrt0CheckBox.setToolTipText(BootloaderResourceBundle.getString("WRT0_TEXT"));
        wrt0CheckBox.addItemListener(this);
        add(wrt0CheckBox);
	  }
	  else
	    wrt0CheckBox = null;

	  if (device.getConfigBits().isWrtd()) {
        wrtdCheckBox = new JCheckBox("WRTD");
        wrtdCheckBox.setToolTipText(BootloaderResourceBundle.getString("WRTD_TEXT"));
        wrtdCheckBox.addItemListener(this);
        add(wrtdCheckBox);
	  }
	  else
	    wrtdCheckBox = null;

	  if (device.getConfigBits().isWrtb()) {
        wrtbCheckBox = new JCheckBox("WRTB");
        wrtbCheckBox.setToolTipText(BootloaderResourceBundle.getString("WRTB_TEXT"));
        wrtbCheckBox.addItemListener(this);
        add(wrtbCheckBox);
	  }
	  else
	    wrtbCheckBox = null;

	  if (device.getConfigBits().isWrtc()) {
        wrtcCheckBox = new JCheckBox("WRTC");
        wrtcCheckBox.setToolTipText(BootloaderResourceBundle.getString("WRTC_TEXT"));
        wrtcCheckBox.addItemListener(this);
        add(wrtcCheckBox);
	  }
	  else
	    wrtcCheckBox = null;
    } // Config6Panel

    private void selectConfig() {
      int conf = bootloader.getConfig(device.loConfigMem +
                                      CONFIG6L) & 0xff;

      if (wrt7CheckBox != null) {
        if ((conf & (1 << 7)) > 0)
          wrt7CheckBox.setSelected(true);
        else
          wrt7CheckBox.setSelected(false);
      }
      
      if (wrt6CheckBox != null) {
        if ((conf & (1 << 6)) > 0)
          wrt6CheckBox.setSelected(true);
        else
          wrt6CheckBox.setSelected(false);
      }
      
      if (wrt5CheckBox != null) {
        if ((conf & (1 << 5)) > 0)
          wrt5CheckBox.setSelected(true);
        else
          wrt5CheckBox.setSelected(false);
      }
      
      if (wrt4CheckBox != null) {
        if ((conf & (1 << 4)) > 0)
          wrt4CheckBox.setSelected(true);
        else
          wrt4CheckBox.setSelected(false);
      }

      if (wrt3CheckBox != null) {
        if ((conf & (1 << 3)) > 0)
          wrt3CheckBox.setSelected(true);
        else
          wrt3CheckBox.setSelected(false);
      }
      
      if (wrt2CheckBox != null) {
        if ((conf & (1 << 2)) > 0)
          wrt2CheckBox.setSelected(true);
        else
          wrt2CheckBox.setSelected(false);
      }
      
      if (wrt1CheckBox != null) {
        if ((conf & (1 << 1)) > 0)
          wrt1CheckBox.setSelected(true);
        else
          wrt1CheckBox.setSelected(false);
      }
      
      if (wrt0CheckBox != null) {
        if ((conf & 1) > 0)
          wrt0CheckBox.setSelected(true);
        else
          wrt0CheckBox.setSelected(false);
      }
      
      conf = bootloader.getConfig(device.loConfigMem + CONFIG6H) & 0xff;

      if (wrtdCheckBox != null) {
          if ((conf & (1 << 7)) > 0)
            wrtdCheckBox.setSelected(true);
          else
            wrtdCheckBox.setSelected(false);
      }
      
      if (wrtbCheckBox != null) {
        if ((conf & (1 << 6)) > 0)
          wrtbCheckBox.setSelected(true);
        else
          wrtbCheckBox.setSelected(false);
      }
      
      if (wrtcCheckBox != null) {
        if ((conf & (1 << 5)) > 0)
          wrtcCheckBox.setSelected(true);
        else
          wrtcCheckBox.setSelected(false);
      }
   } // selectConfig

    /**
    Event handler for changes in the current selection of the Choices.
    */
    public void itemStateChanged(ItemEvent e) {
      // Set the parameters in the panel.
      setConfiguration();
    } // itemStateChanged
  } // Config6Panel

  class Config7Panel extends JPanel implements ItemListener {
	final static long serialVersionUID = 0;

    public Config7Panel() {
      setBorder(BorderFactory.
                createTitledBorder(BorderFactory.createEtchedBorder(),
                "CONFIG7L & CONFIG7H"));

      if (device.getConfigBits().isEbtr7()) {
        ebtr7CheckBox = new JCheckBox("EBTR7");
        ebtr7CheckBox.setToolTipText(BootloaderResourceBundle.getString("EBTR7_TEXT"));
        ebtr7CheckBox.addItemListener(this);
        add(ebtr7CheckBox);
      }
      else
      	ebtr7CheckBox = null;
      
      if (device.getConfigBits().isEbtr6()) {
        ebtr6CheckBox = new JCheckBox("EBTR6");
        ebtr6CheckBox.setToolTipText(BootloaderResourceBundle.getString("EBTR6_TEXT"));
        ebtr6CheckBox.addItemListener(this);
        add(ebtr6CheckBox);
      }
      else
      	ebtr6CheckBox = null;

      if (device.getConfigBits().isEbtr5()) {
        ebtr5CheckBox = new JCheckBox("EBTR5");
        ebtr5CheckBox.setToolTipText(BootloaderResourceBundle.getString("EBTR5_TEXT"));
        ebtr5CheckBox.addItemListener(this);
        add(ebtr5CheckBox);
      }
      else
    	ebtr5CheckBox = null;

      if (device.getConfigBits().isEbtr4()) {
        ebtr4CheckBox = new JCheckBox("EBTR4");
        ebtr4CheckBox.setToolTipText(BootloaderResourceBundle.getString("EBTR4_TEXT"));
        ebtr4CheckBox.addItemListener(this);
        add(ebtr4CheckBox);
      }
      else
        ebtr4CheckBox = null;

	  if (device.getConfigBits().isEbtr3()) {
        ebtr3CheckBox = new JCheckBox("EBTR3");
        ebtr3CheckBox.setToolTipText(BootloaderResourceBundle.getString("EBTR3_TEXT"));
        ebtr3CheckBox.addItemListener(this);
        add(ebtr3CheckBox);
	  }
	  else
	    ebtr3CheckBox = null;

	  if (device.getConfigBits().isEbtr2()) {
        ebtr2CheckBox = new JCheckBox("EBTR2");
        ebtr2CheckBox.setToolTipText(BootloaderResourceBundle.getString("EBTR2_TEXT"));
        ebtr2CheckBox.addItemListener(this);
        add(ebtr2CheckBox);
	  }
	  else
	  	ebtr2CheckBox = null;
	  	
	  if (device.getConfigBits().isEbtr1()) {
          ebtr1CheckBox = new JCheckBox("EBTR1");
          ebtr1CheckBox.setToolTipText(BootloaderResourceBundle.getString("EBTR1_TEXT"));
          ebtr1CheckBox.addItemListener(this);
          add(ebtr1CheckBox);
	  }
	  else
	  	ebtr1CheckBox = null;

	  if (device.getConfigBits().isEbtr0()) {
        ebtr0CheckBox = new JCheckBox("EBTR0");
        ebtr0CheckBox.setToolTipText(BootloaderResourceBundle.getString("EBTR0_TEXT"));
        ebtr0CheckBox.addItemListener(this);
        add(ebtr0CheckBox);
	  }
	  else
	  	ebtr0CheckBox = null;

	  if (device.getConfigBits().isEbtr()) {
        ebtrCheckBox = new JCheckBox("EBTR");
        ebtrCheckBox.setToolTipText(BootloaderResourceBundle.getString("EBTR0_TEXT"));
        ebtrCheckBox.addItemListener(this);
        add(ebtrCheckBox);
	  }
	  else
	  	ebtrCheckBox = null;

	  if (device.getConfigBits().isEbtrb()) {
          ebtrbCheckBox = new JCheckBox("EBTRB");
          ebtrbCheckBox.setToolTipText(BootloaderResourceBundle.getString("EBTRB_TEXT"));
          ebtrbCheckBox.addItemListener(this);
          add(ebtrbCheckBox);
	  }
	  else
	  	ebtrbCheckBox = null;
    } // Config7Panel

    private void selectConfig() {
      int conf = bootloader.getConfig(device.loConfigMem +
                                      CONFIG7L) & 0xff;

      if (ebtr7CheckBox != null) {
        if ((conf & (1 << 7)) > 0)
          ebtr7CheckBox.setSelected(true);
        else
          ebtr7CheckBox.setSelected(false);

        if ((conf & (1 << 6)) > 0)
          ebtr6CheckBox.setSelected(true);
        else
          ebtr6CheckBox.setSelected(false);

        if ((conf & (1 << 5)) > 0)
          ebtr5CheckBox.setSelected(true);
        else
          ebtr5CheckBox.setSelected(false);

        if ((conf & (1 << 4)) > 0)
          ebtr4CheckBox.setSelected(true);
        else
          ebtr4CheckBox.setSelected(false);
      }

      if (ebtr3CheckBox != null) {
        if ((conf & (1 << 3)) > 0)
          ebtr3CheckBox.setSelected(true);
        else
          ebtr3CheckBox.setSelected(false);
      }
      
      if (ebtr2CheckBox != null) {
        if ((conf & (1 << 2)) > 0)
          ebtr2CheckBox.setSelected(true);
        else
          ebtr2CheckBox.setSelected(false);
      }
      
      if (ebtr1CheckBox != null) {
        if ((conf & (1 << 1)) > 0)
          ebtr1CheckBox.setSelected(true);
        else
          ebtr1CheckBox.setSelected(false);
      }
      
      if (ebtr0CheckBox != null) {
        if ((conf & 1) > 0)
          ebtr0CheckBox.setSelected(true);
        else
          ebtr0CheckBox.setSelected(false);
      }
      
      if (ebtrCheckBox != null) {
        if ((conf & 1) > 0)
          ebtrCheckBox.setSelected(true);
        else
          ebtrCheckBox.setSelected(false);
      }
      
      conf = bootloader.getConfig(device.loConfigMem + CONFIG7H) & 0xff;

      if ((conf & (1 << 6)) > 0)
        ebtrbCheckBox.setSelected(true);
      else
        ebtrbCheckBox.setSelected(false);
    } // selectConfig

    /**
    Event handler for changes in the current selection of the Choices.
    */
    public void itemStateChanged(ItemEvent e) {
      // Set the parameters in the panel.
      setConfiguration();
    } // itemStateChanged
   } // Config7Panel

  /**
  Handles events from the OK button. When OK is pressed the dialog becomes
  invisible, disposes of its self, and retruns.
  */
  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    int i;

    if (cmd.equals(BootloaderResourceBundle.getString("OK_TEXT"))) {
      // Store config
      for (i = device.loConfigMem; i <= device.hiConfigMem; i++)
        bootloader.loadConfig(i, config[i - device.loConfigMem]);
    }

    setVisible(false);
    dispose();
  } // actionPerformed
} // ConfigDialog
