/*
 * @(#)EeDataDialog.java	0.1 02/10/04 SMI
 *
 * Copyright (c) 2002 Martin Dubuc All Rights Reserved.
 *
 * Martin Dubuc grants you ("Licensee") a non-exclusive, royalty free, license
 * to use, modify and redistribute this software in source and binary
 * code form, provided that i) this copyright notice and license appear
 * on all copies of the software; and ii) Licensee does not utilize the
 * software in a manner which is disparaging to Martin Dubuc.
 *
 * This software is provided "AS IS," without a warranty of any kind.
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. MARTIN DUBUC AND
 * ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE
 * SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL MARTIN DUBUC OR ITS LICENSORS
 * BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING
 * OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF MARTIN DUBUC HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control
 * of aircraft, air traffic, aircraft navigation or aircraft
 * communications; or in the design, construction, operation or
 * maintenance of any nuclear facility. Licensee represents and
 * warrants that it will not use or redistribute the Software for such
 * purposes.
 */

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;

/**
A single response modal about dialog. When the OK button is pressed the dialog returns.
*/
public class EeDataDialog extends JDialog implements ActionListener {
  final static long serialVersionUID = 0;
  
  final private int WIDTH = 500;
  final private int HEIGHT = 500;
  final private int ADDRESS_WIDTH = 60;
  final private int CELL_WIDTH = 30;
  final private int ADDRESS_COLUMN = 0;

  final private int NUM_DATA_COLS = 8;

  private JTable eeDataTable;

  private Bootloader bootloader;
  private PicDevice device;

  /**
  Creates a new <code>ConfigDialog</code>.

  @param parent Any JFrame.
  @param connection The serial connection.
  */
  public EeDataDialog(Bootloader bootloader, SerialConnection connection) {
    super(bootloader, true);
    String[] columnNames = { BootloaderResourceBundle.getString("ADDRESS_TEXT"), "+0", "+1", "+2", "+3", 
                             "+4", "+5", "+6", "+7" };
    Object[][] cells;
    int i, j;
    int numRows;

    Container contentPane = getContentPane();

    JPanel panel = new JPanel();

    this.bootloader = bootloader;

    device = bootloader.getDevice();

    numRows = (device.hiEeData - device.loEeData + 1) / NUM_DATA_COLS;
    cells = new Object[numRows][NUM_DATA_COLS + 1];
 
    for (i = 0; i < numRows; i++) {
      cells[i][0] = new String(Utils.hexToAscii((i * NUM_DATA_COLS) >> 8) +
                               Utils.hexToAscii((i * NUM_DATA_COLS) & 0xff));
      for (j = 0; j < NUM_DATA_COLS; j++)
        cells[i][j + 1] =
          new String(Utils.hexToAscii(bootloader.getEeData(device.loEeData +
                                                           i * NUM_DATA_COLS + j)));
    }

    DefaultTableModel defaultTableModel = new DefaultTableModel(cells,
                                                                columnNames);
    EeDataTableModel eeDataTableModel = new EeDataTableModel(defaultTableModel);

    eeDataTable = new JTable(eeDataTableModel);
    eeDataTable.setRowSelectionAllowed(false);
    eeDataTable.setColumnSelectionAllowed(false);

    // Set column size
    TableColumnModel columnModel = eeDataTable.getColumnModel();
    TableColumn column;
 
    for (i = 0; i < NUM_DATA_COLS + 1; i++) {
      column = columnModel.getColumn(i);
      column.setResizable(false);
      //column.setHeaderRenderer(new DataHeaderRenderer());

      if (i == ADDRESS_COLUMN) {
        column.setCellRenderer(eeDataTable.getTableHeader().getDefaultRenderer());
        column.setPreferredWidth(ADDRESS_WIDTH);
      }
      else {
        column.setPreferredWidth(CELL_WIDTH);
      }
    }

    JScrollPane pane = new JScrollPane(eeDataTable);

    panel.add(pane);

    contentPane.add(panel, "Center");

    JPanel buttonPanel = new JPanel();
    JButton okButton = new JButton(BootloaderResourceBundle.getString("OK_TEXT"));
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    JButton cancelButton = new JButton(BootloaderResourceBundle.getString("CANCEL_TEXT"));
    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);
    contentPane.add(buttonPanel, "South");

    FontMetrics fm = getFontMetrics(getFont());

    setSize(WIDTH, HEIGHT);
    setLocation(bootloader.getLocationOnScreen().x + 30, 
		        bootloader.getLocationOnScreen().y + 30);
		        
	setTitle(BootloaderResourceBundle.getString("EDIT_EEPROM_DATA_TEXT"));

    setVisible(true);
    setResizable(false);
  } // EeDataDialog

  class EeDataTableModel extends AbstractTableModel {
	final static long serialVersionUID = 0;

    public EeDataTableModel(TableModel m) {
      model = m;
    } // EeDataTableModel

    public boolean isCellEditable(int r, int c) {
      if (c == 0) // The address column is read-only
        return false;
      return true;
    } // isCellEditable

    // Delegate all remaining routines to the model
    public Object getValueAt(int row, int column) {
      return model.getValueAt(row, column);
    } // getValueAt

    public void setValueAt(Object value, int row, int column) {
      model.setValueAt(value, row, column);
    } // setValueAt

    public int getRowCount() {
      return model.getRowCount();
    } // getRowCount

    public int getColumnCount() {
      return model.getColumnCount();
    } // getColumnCount

    public String getColumnName(int column) {
      return model.getColumnName(column);
    } // getColumnName

    public Class getColumnClass(int column) {
      return model.getColumnClass(column);
    } // getRowCount

    private TableModel model;
  } // EeDataTableModel

  class DataHeaderCellRenderer implements TableCellRenderer {
    public Component getTableCellRendererComponent(final JTable table,
                                                   final Object value,                                                
                                                   final boolean isSelected,
                                                   final boolean cellHasFocus,
                                                   final int row,
                                                   final int column) {
      return new JPanel() {
    	final static long serialVersionUID = 0;

        public void paintComponent(Graphics g) {
          Font font = new Font("Courier", Font.BOLD, 12);
          String text = value.toString();
          FontMetrics fm = g.getFontMetrics(font);
          g.setColor(table.getBackground());
          g.fillRect(0, 0, getWidth(), getHeight());
          g.setColor(table.getForeground());
          g.setFont(font);
          g.drawString(text, 0, fm.getAscent());
        }
      };
    } // getTableCellRendererComponent  
  }  // DataHeaderCellRenderer

  /**
  Handles events from the OK/Cancel button. When OK/Cancel is pressed the
  dialog becomes invisible, disposes of its self, and retruns.
  */
  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    int i;
    Object obj;
    int val;

    if (cmd.equals("OK")) {
      // Store EEPROM Data
      for (i = device.loEeData; i <= device.hiEeData; i++) {
        obj = eeDataTable.getValueAt((i - device.loEeData) / NUM_DATA_COLS,
                                     ((i - device.loEeData) % NUM_DATA_COLS) + 1);
        val = Integer.parseInt(obj.toString(), 16);
        bootloader.loadData(i, (byte) val);
      }
    }

    setVisible(false);
    dispose();
  } // actionPerformed
} // EeDataDialog
