/*
 * @(#)HelpDialog.java	0.1 02/11/25 SMI
 *
 * Copyright (c) 2002 Martin Dubuc All Rights Reserved.
 *
 * Martin Dubuc grants you ("Licensee") a non-exclusive, royalty free, license
 * to use, modify and redistribute this software in source and binary
 * code form, provided that i) this copyright notice and license appear
 * on all copies of the software; and ii) Licensee does not utilize the
 * software in a manner which is disparaging to Martin Dubuc.
 *
 * This software is provided "AS IS," without a warranty of any kind.
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. MARTIN DUBUC AND
 * ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE
 * SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL MARTIN DUBUC OR ITS LICENSORS
 * BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING
 * OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF MARTIN DUBUC HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control
 * of aircraft, air traffic, aircraft navigation or aircraft
 * communications; or in the design, construction, operation or
 * maintenance of any nuclear facility. Licensee represents and
 * warrants that it will not use or redistribute the Software for such
 * purposes.
 */

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.text.html.*;
import java.net.URL;

/**
A single response modal about dialog. When the OK button is pressed the dialog returns.
*/
public class HelpDialog extends JDialog implements ActionListener {
  final static long serialVersionUID = 0;

  final private int WIDTH = 600;
  final private int HEIGHT = 500;

  /**
  Creates a new <code>AboutDialog</code>.

  @param parent Any JFrame.
  */
  public HelpDialog(JFrame parent) {
    super(parent, true);

    int width = WIDTH;
    int height = HEIGHT;

    JEditorPane helpPane = new JEditorPane();

    Container contentPane = getContentPane();

    helpPane.setEditable(false);
    helpPane.setFont(new Font("Arial", Font.PLAIN, 12));
    helpPane.addHyperlinkListener(new HyperlinkListener() {
      public void hyperlinkUpdate(HyperlinkEvent e) {
        if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
          JEditorPane pane = (JEditorPane) e.getSource();
          if (e instanceof HTMLFrameHyperlinkEvent) {
            HTMLFrameHyperlinkEvent evt = (HTMLFrameHyperlinkEvent) e;
            HTMLDocument doc = (HTMLDocument) pane.getDocument();
            doc.processHTMLFrameHyperlinkEvent(evt);
          }
          else {
            try {
 	          pane.setPage(e.getURL());
 	        }
 	        catch (Throwable t) {
              t.printStackTrace();
            }
          }
        }
      }
    });

    URL url;
    
    if (BootloaderResourceBundle.getLanguage() == BootloaderParameters.LANGUAGE_FRENCH) 
	  url = getClass().getResource("help_fr.html");
	else
	  url = getClass().getResource("help.html");

    try {
      helpPane.setPage(url);
    }
    catch(IOException e) {
      helpPane.setText("Error: " + e);
    }

    JScrollPane scrollPane = new JScrollPane(helpPane);
    scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    contentPane.add(scrollPane);

    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

    setSize(WIDTH, HEIGHT);
    setLocation(screenSize.width / 2 - width / 2, 
                screenSize.height / 2 - height / 2);
    
    if (BootloaderResourceBundle.getLanguage() == BootloaderParameters.LANGUAGE_FRENCH)
	  setTitle(BootloaderResourceBundle.getString("HELP_FOR_TEXT") + " " + Bootloader.PRODUCT_NAME);
	else
	  setTitle(Bootloader.PRODUCT_NAME + " " + BootloaderResourceBundle.getString("HELP_FOR_TEXT"));

    setVisible(true);
  } // HelpDialog
  
  /**
  Handles events from the OK button. When OK is pressed the dialog becomes
  invisible, disposes of its self, and retruns.
  */
  public void actionPerformed(ActionEvent e) {
    setVisible(false);
    dispose();
  } // actionPerformed
} // HelpDialog
