/* @(#)PicDevice.java	0.1 02/09/27 SMI
 *
 * Copyright (c) 2002-2004 Martin Dubuc All Rights Reserved.
 *
 * Martin Dubuc grants you ("Licensee") a non-exclusive, royalty free, license
 * to use, modify and redistribute this software in source and binary
 * code form, provided that i) this copyright notice and license appear
 * on all copies of the software; and ii) Licensee does not utilize the
 * software in a manner which is disparaging to Martin Dubuc.
 *
 * This software is provided "AS IS," without a warranty of any kind.
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. MARTIN DUBUC
 * AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE
 * SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL MARTIN DUBUC OR ITS LICENSORS
 * BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING
 * OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF MARTIN DUBUC
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control
 * of aircraft, air traffic, aircraft navigation or aircraft
 * communications; or in the design, construction, operation or
 * maintenance of any nuclear facility. Licensee represents and
 * warrants that it will not use or redistribute the Software for such
 * purposes.
 */

import java.util.StringTokenizer;

/**
A class that handles the details of a PIC device.
*/
public class PicDevice {
  public final static int MAX_PROGRAM_MEMORY_SIZE = 131072;
  public final static int MAX_EEDATA_SIZE = 1024;
  public final static int CONFIG_SIZE = 14;

  public final static int CONFIGURATION_ADDRESS_LO = 0x300000;
  public final static int CONFIGURATION_ADDRESS_HI = 0x30000D;

  public final static int DEVICE_ID_BASE_ADDRESS = 0x3FFFFE;
  
  public String name;

  public int loProgramMem;
  public int hiProgramMem;
  public int loConfigMem;
  public int hiConfigMem;
  public int loEeData;
  public int hiEeData;

  public int writeBlock;
  public int readBlock;
  public int eraseBlock;

  public ConfigurationBits configBits;
  
  public PicDevice alternateDevice;
  
  /**
  Creates a PicDevice object and initializes variables passed in
  as params.

  @param name The name of the PIC.
  @param loProgramMem The low address of program memory.
  @param hiProgramMem The high address of program memory.
  @param loConfigMem The low address of configuration memory.
  @param hiConfigMem The high address of configuration memory.
  @param loEeDataMem The low address of EE data memory.
  @param hiEeDataMem The high address of EE datamemory.
  */
  public PicDevice(String name,
                   int loProgramMem, int hiProgramMem,
                   int loEeData, int hiEeData, String configBits) {
    this.name = new String(name);
    this.loProgramMem = loProgramMem;
    this.hiProgramMem = hiProgramMem;
    this.loConfigMem = CONFIGURATION_ADDRESS_LO;
    this.hiConfigMem = CONFIGURATION_ADDRESS_HI;
    this.loEeData = loEeData;
    this.hiEeData = hiEeData;

    this.configBits = new ConfigurationBits(configBits);
    
    alternateDevice = null;
    
    initBlockLengths();
  } // PicDevice

  /**
  Init write, read and erase block lengths to a default value.
  */
  private void initBlockLengths() {
    writeBlock = 8;
    readBlock = 1;
    eraseBlock = 64;
  } // initBlockLengths

  /**
  Creates a PicDevice object and initializes variables passed in
  as params.

  @param name The name of the PIC.
  @param propertyString The property string associated with PIC.
  */
  public PicDevice(String name, String propertyString) {
    StringTokenizer st = new StringTokenizer(propertyString, ",");
    String str;

    this.name = name;
    this.loConfigMem = CONFIGURATION_ADDRESS_LO;
    this.hiConfigMem = CONFIGURATION_ADDRESS_HI;
    
    alternateDevice = null;

    initBlockLengths();

    // Extract the fields from the property string.

    // Skip Device ID.
    if (!st.hasMoreTokens())
      return;

    str = st.nextToken();

    // Extract high and low program memory addresses
    if (!st.hasMoreTokens())
      return;

    str = st.nextToken();
    loProgramMem = Integer.parseInt(str, 16);

    if (!st.hasMoreTokens())
      return;

    str = st.nextToken();
    hiProgramMem = Integer.parseInt(str, 16);

    // Extract high and low EEPROM data addresses
    if (!st.hasMoreTokens())
      return;

    str = st.nextToken();
    loEeData = Integer.parseInt(str, 16);

    if (!st.hasMoreTokens())
      return;

    str = st.nextToken();
    hiEeData = Integer.parseInt(str, 16);
    
    // Read Configuration bits
    String config = new String();
    
    while (st.hasMoreTokens()) {
    	str = st.nextToken();
    	config = config.concat(str);
    	if (st.hasMoreTokens())
    		config = config.concat(",");
    }
    
    configBits = new ConfigurationBits(config);
  } // PicDevice
  
  public ConfigurationBits getConfigBits() {
  	return configBits;
  } // getConfigBits
  
  public String getConfigBitsString() {
  	return configBits.toString();
  } // getConfigBitsString
}
