/* @(#)PicDeviceTable.java	0.1 15/07/03 SMI
 *
 * Copyright (c) 2002, 2003 Martin Dubuc All Rights Reserved.
 *
 * Martin Dubuc grants you ("Licensee") a non-exclusive, royalty free, license
 * to use, modify and redistribute this software in source and binary
 * code form, provided that i) this copyright notice and license appear
 * on all copies of the software; and ii) Licensee does not utilize the
 * software in a manner which is disparaging to Martin Dubuc.
 *
 * This software is provided "AS IS," without a warranty of any kind.
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. MARTIN DUBUC AND
 * ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE
 * SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL MARTIN DUBUC OR ITS LICENSORS
 * BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING
 * OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF MARTIN DUBUC HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control
 * of aircraft, air traffic, aircraft navigation or aircraft
 * communications; or in the design, construction, operation or
 * maintenance of any nuclear facility. Licensee represents and
 * warrants that it will not use or redistribute the Software for such
 * purposes.
 */

import java.io.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import java.util.StringTokenizer;

/**
A class that stores PIC device properties.
*/
public class PicDeviceTable {
  private static final String PROPERTY_FILENAME = "PicDevices.properties";

  private Hashtable<Integer, PicDevice> deviceTable;

  /**
  Default constructor. 
  */
  public PicDeviceTable() {
    deviceTable =  new Hashtable<Integer, PicDevice>();
  } // PicDeviceProperties

  public PicDevice get(int deviceId) {
    PicDevice device;

    device = (PicDevice) deviceTable.get(new Integer(deviceId));

    return device;
  } // get

  // This routine is called if the PicDevices properties file does not exist to
  // fill a new properties file with known devices.
  public void initPicDevices() {
  	PicDevice device;
  	
    // Fill in device table with known PIC devices
    device = new PicDevice("PIC18F252", 0x200, 0x7FFF, 0, 255,
                           "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB");
	device.alternateDevice =
		new PicDevice("PIC18F2539", 0x200, 0x5FFF, 0, 255,
					  "FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,DEBUG,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB");
    deviceTable.put(new Integer(32), device);

    device = new PicDevice("PIC18F452", 0x200, 0x7FFF, 0, 255,
                           "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"); 
	device.alternateDevice =
        new PicDevice("PIC18F4539", 0x200, 0x5FFF, 0, 255, 
					  "FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,DEBUG,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB");
	deviceTable.put(new Integer(33), device);

    device = new PicDevice("PIC18F242", 0x200, 0x3FFF, 0, 255,
                           "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB");
	device.alternateDevice =
		new PicDevice("PIC18F2439", 0x200, 0x2FFF, 0, 255,
			          "FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,DEBUG,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB");
    deviceTable.put(new Integer(36), device);
    		                                   
    device = new PicDevice("PIC18F442", 0x200, 0x3FFF, 0, 255,
                           "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB");
	device.alternateDevice =
	    new PicDevice("PIC18F4439", 0x200, 0x2FFF, 0, 255,
				      "FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,DEBUG,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB");
    deviceTable.put(new Integer(37), device);

    deviceTable.put(new Integer(40),
                    new PicDevice("PIC18F2320", 0x200, 0, 0, 255,
                                  "IESO,FCMEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,MCLRE,PBADEN,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(41),
                    new PicDevice("PIC18F4320", 0x200, 0x1FFF, 0, 255,
                                  "IESO,FCMEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,MCLRE,PBADEN,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(44),
                    new PicDevice("PIC18F2220", 0x200, 0x0FFF, 0, 255,
                                  "IESO,FCMEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,MCLRE,PBADEN,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(45),
                    new PicDevice("PIC18F4220", 0x200, 0x0FFF, 0, 255,
                                  "IESO,FCMEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,MCLRE,PBADEN,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(48),
                    new PicDevice("PIC18F8720", 0x200, 0x1FFFF, 0, 1023,
                                  "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,WAIT,PM,CCP2MX,DEBUG,LVP,STVREN,CP7,CP6,CP5,CP4,CP3,CP2,CP1,CP0,CPD,CPB,WRT7,WRT6,WRT5,WRT4,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR7,EBTR6,EBTR5,EBTR4,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(49),
                    new PicDevice("PIC18F6720", 0x200, 0x1FFFF, 0, 1023,
                                  "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,CCP2MX,DEBUG,LVP,STVREN,CP7,CP6,CP5,CP4,CP3,CP2,CP1,CP0,CPD,CPB,WRT7,WRT6,WRT5,WRT4,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR7,EBTR6,EBTR5,EBTR4,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));

    device = new PicDevice("PIC18F8620", 0x200, 0xFFFF, 0, 1023, 
                           "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,WAIT,PM,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB");
    device.alternateDevice =
        new PicDevice("PIC18F8490", 0x200, 0x7FFF, 0, 1023,
                      "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,WAIT,BW,PM,MCLRE,LPT1OSC,CCP2MX,DEBUG,XINST,STVREN,CP,EBTR");
    deviceTable.put(new Integer(52), device);

    device = new PicDevice("PIC18F6620", 0x200, 0xFFFF, 0, 1023,
                           "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB");
    device.alternateDevice =
        new PicDevice("PIC18F6490", 0x200, 0x7FFF, 0, 1023,
                      "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,WAIT,BW,PM,MCLRE,LPT1OSC,CCP2MX,DEBUG,XINST,STVREN,CP");
    deviceTable.put(new Integer(53), device);

    deviceTable.put(new Integer(54),
			        new PicDevice("PIC18F8410", 0x800, 0x3FFF, 0, 0,
			                      "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,WAIT,BW,PM,MCLRE,LPT1OSC,CCP2MX,DEBUG,XINST,STVREN,CP,EBTR"));
	deviceTable.put(new Integer(55),
                    new PicDevice("PIC18F6410", 0x800, 0x3FFF, 0, 0,
                                  "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,WAIT,BW,PM,MCLRE,LPT1OSC,CCP2MX,DEBUG,XINST,STVREN,CP"));
    deviceTable.put(new Integer(62),
                    new PicDevice("PIC18F1320", 0x200, 0x1FFF, 0, 255,
                                  "IESO,FSCM,FOSC30,BORV,BOREN,PWRTEN,WDTPS30,WDTEN,MCLRE,DEBUG,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(63),
                    new PicDevice("PIC18F1220", 0x200, 0x0FFF, 0, 255,
                                  "IESO,FSCM,FOSC30,BORV,BOREN,PWRTEN,WDTPS30,WDTEN,MCLRE,DEBUG,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(64),
                    new PicDevice("PIC18F248", 0x200, 0x3FFF, 0, 255,
                                  "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(65),
                    new PicDevice("PIC18F448", 0x200, 0x3FFF, 0, 255,
                                  "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(66),
                    new PicDevice("PIC18F258", 0x200, 0x7FFF, 0, 255,
                                  "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(67),
                    new PicDevice("PIC18F458", 0x200, 0x7FFF, 0, 255,
                                  "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(68),
					new PicDevice("PIC18F4431", 0x200, 0x3FFF, 0, 255,
								  "IESO,FSCM,FOSC30,BORV,BOREN,PWRTEN,WINEN,WDTPS20,WDTEN,!T1OSCMX,HPOL,LPOL,PWMPIN,MCLRE,EXCLKMX,PWM4MX,SSPMX,FLTAMX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(69),
					new PicDevice("PIC18F4331", 0x200, 0x1FFF, 0, 255,
								  "IESO,FSCM,FOSC30,BORV,BOREN,PWRTEN,WINEN,WDTPS20,WDTEN,!T1OSCMX,HPOL,LPOL,PWMPIN,MCLRE,EXCLKMX,PWM4MX,SSPMX,FLTAMX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(70),
					new PicDevice("PIC18F2431", 0x200, 0x3FFF, 0, 255,
							      "IESO,FSCM,FOSC30,BORV,BOREN,PWRTEN,WINEN,WDTPS20,WDTEN,!T1OSCMX,HPOL,LPOL,PWMPIN,MCLRE,EXCLKMX,PWM4MX,SSPMX,FLTAMX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(71),
                    new PicDevice("PIC18F2331", 0x200, 0x1FFF, 0, 255,
                                  "IESO,FSCM,FOSC30,BORV,BOREN,PWRTEN,WINEN,WDTPS20,WDTEN,!T1OSCMX,HPOL,LPOL,PWMPIN,MCLRE,EXCLKMX,PWM4MX,SSPMX,FLTAMX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(80),
					new PicDevice("PIC18F8680", 0x800, 0xFFFF, 0, 1023,
								  "OSCSEN,FOSC30,BORV,BOREN,PWRTEN,WDTPS30,WDTEN,WAIT,PM,MCLRE,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));									  
	deviceTable.put(new Integer(81),
					new PicDevice("PIC18F6680", 0x800, 0xFFFF, 0, 1023,
								  "OSCSEN,FOSC30,BORV,BOREN,PWRTEN,WDTPS30,WDTEN,MCLRE,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(82),
					new PicDevice("PIC18F8585", 0x800, 0xBFFF, 0, 1023,
								  "OSCSEN,FOSC30,BORV,BOREN,PWRTEN,WDTPS30,WDTEN,WAIT,PM,MCLRE,CCP2MX,DEBUG,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(83),
                    new PicDevice("PIC18F6585", 0x800, 0xBFFF, 0, 1023,
                                  "OSCSEN,FOSC30,BORV,BOREN,PWRTEN,WDTPS30,WDTEN,MCLRE,CCP2MX,DEBUG,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(84),
					new PicDevice("PIC18F8621", 0x200, 0xFFFF, 0, 1023,
								  "OSCSEN,FOSC30,BORV,BOREN,PWRTEN,WDTPS30,WDTEN,WAIT,PM,MCLRE,ECCPMX,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(85),
                    new PicDevice("PIC18F6621", 0x800, 0xFFFF, 0, 1023,
                                  "OSCSEN,FOSC30,BORV,BOREN,PWRTEN,WDTPS30,WDTEN,MCLRE,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
    deviceTable.put(new Integer(86),
                    new PicDevice("PIC18F8525", 0x800, 0xBFFF, 0, 1023,
                                  "OSCSEN,FOSC30,BORV,BOREN,PWRTEN,WDTPS30,WDTEN,WAIT,PM,MCLRE,ECCPMX,CCP2MX,DEBUG,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(87),
					new PicDevice("PIC18F6525", 0x800, 0xBFFF, 0, 1023,
								  "OSCSEN,FOSC30,BORV,BOREN,PWRTEN,WDTPS30,WDTEN,MCLRE,CCP2MX,DEBUG,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(88),
					new PicDevice("PIC18F8520", 0x800, 0x7FFF, 0, 1023,
								  "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,WAIT,PM,T1OSCMX,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(89),
					new PicDevice("PIC18F6520", 0x800, 0x7FFF, 0, 1023,
								  "OSCSEN,FOSC20,BORV,BOREN,PWRTEN,WDTPS20,WDTEN,T1OSCMX,CCP2MX,DEBUG,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(92),
			        new PicDevice("PIC18F8390", 0x200, 0x3FFF, 0, 1023,
			                      "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,WAIT,BW,PM,MCLRE,LPT1OSC,CCP2MX,DEBUG,XINST,STVREN,CP,EBTR"));
	deviceTable.put(new Integer(93),
					new PicDevice("PIC18F6390", 0x200, 0x3FFF, 0, 1023,
                                  "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,WAIT,BW,PM,MCLRE,LPT1OSC,CCP2MX,DEBUG,XINST,STVREN,CP"));
    deviceTable.put(new Integer(94),
			        new PicDevice("PIC18F8310", 0x800, 0x1FFF, 0, 1023,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,WAIT,BW,PM,MCLRE,LPT1OSC,CCP2MX,DEBUG,XINST,STVREN,CP,EBTR"));
	deviceTable.put(new Integer(95),
			        new PicDevice("PIC18F6310", 0x800, 0x1FFF, 0, 1023,
					              "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,WAIT,BW,PM,MCLRE,LPT1OSC,CCP2MX,DEBUG,XINST,STVREN,CP"));
	deviceTable.put(new Integer(96),
			        new PicDevice("PIC18F4620", 0x800, 0xFFFF, 0, 1023,
			                      "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(97),
                    new PicDevice("PIC18F4610", 0x800, 0xFFFF, 0, 0,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(98),
                    new PicDevice("PIC18F4525", 0x800, 0xBFFF, 0, 1023,
                                  "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(99),
                    new PicDevice("PIC18F4515", 0x800, 0xBFFF, 0, 0,
                                  "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(100),
                    new PicDevice("PIC18F2620", 0x800, 0xFFFF, 0, 1023,
	                              "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(101),
	                new PicDevice("PIC18F2610", 0x800, 0xFFFF, 0, 0,
                                  "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(102),
                    new PicDevice("PIC18F2525", 0x800, 0xBFFF, 0, 1023,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(103),
                    new PicDevice("PIC18F2515", 0x800, 0xBFFF, 0, 0,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(116),
                    new PicDevice("PIC18F4680", 0x800, 0xFFFF, 0, 1023,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,DEBUG,XINST,BBSIZ,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(117),
                    new PicDevice("PIC18F4585", 0x800, 0xBFFF, 0, 1023,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,DEBUG,XINST,BBSIZ,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(118),
                    new PicDevice("PIC18F2680", 0x800, 0xFFFF, 0, 1023,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,DEBUG,XINST,BBSIZ,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(119),
                    new PicDevice("PIC18F2585", 0x800, 0xBFFF, 0, 1023,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,DEBUG,XINST,BBSIZ,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(132),
                    new PicDevice("PIC18F4520", 0x800, 0xFFFF, 0, 255,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(133),
                    new PicDevice("PIC18F4510", 0x800, 0xFFFF, 0, 0,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(134),
                    new PicDevice("PIC18F4420", 0x800, 0x7FFF, 0, 255,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(135),
                    new PicDevice("PIC18F4410", 0x800, 0x7FFF, 0, 0,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(136),
                    new PicDevice("PIC18F2520", 0x800, 0xFFFF, 0, 0,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(137),
                    new PicDevice("PIC18F2510", 0x800, 0xFFFF, 0, 0,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(138),
                    new PicDevice("PIC18F2420", 0x800, 0x7FFF, 0, 0, 
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(139),
                    new PicDevice("PIC18F2410", 0x800, 0x7FFF, 0, 0,
						          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(144),
	        new PicDevice("PIC18F4550", 0x800, 0x7FFF, 0, 255,
	                      "USBPLL,CPUDIV,PLLDIV,IESO,FCMEN,FOSC30,VREGEN,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,ICPRT,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(145),
                    new PicDevice("PIC18F4455", 0x800, 0x7FFF, 0, 255,
						          "USBPLL,CPUDIV,PLLDIV,IESO,FCMEN,FOSC30,VREGEN,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,ICPRT,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(146),
	        new PicDevice("PIC18F2550", 0x800, 0x7FFF, 0, 255,
	                      "USBPLL,CPUDIV,PLLDIV,IESO,FCMEN,FOSC30,VREGEN,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,ICPRT,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(147),
			new PicDevice("PIC18F2455", 0x800, 0x7FFF, 0, 255,
                   		  "USBPLL,CPUDIV,PLLDIV,IESO,FCMEN,FOSC30,VREGEN,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,ICPRT,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(152),
            new PicDevice("PIC18F6627", 0x800, 0x17FFF, 0, 1023,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,CCP2MX,DEBUG,XINST,BBSIZ,LVP,STVREN,CP5,CP4,CP3,CP2,CP1,CP0,CPD,CPB,WRT5,WRT4,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBRT5,EBRT4,EBRT3,EBRT2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(154),
            new PicDevice("PIC18F6527", 0x800, 0xBFFF, 0, 1023,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,CCP2MX,DEBUG,XINST,BBSIZ,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBRT2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(155),
            new PicDevice("PIC18F8527", 0x800, 0xBFFF, 0, 1023,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,WAIT,BW,ABW,PM,MCLRE,LPT1OSC,ECCPMX,CCP2MX,DEBUG,XINST,BBSIZ,LVP,STVREN,CP2,CP1,CP0,CPD,CPB,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBRT2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(156),
            new PicDevice("PIC18F6622", 0x800, 0xFFFF, 0, 1023,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,CCP2MX,DEBUG,XINST,BBSIZ,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBRT3,EBRT2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(157),
            new PicDevice("PIC18F8622", 0x800, 0xFFFF, 0, 1023,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,WAIT,BW,ABW,PM,MCLRE,LPT1OSC,ECCPMX,CCP2MX,DEBUG,XINST,BBSIZ,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBRT3,EBRT2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(159),
            new PicDevice("PIC18F8627", 0x800, 0x17FFF, 0, 1023,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,WAIT,BW,ABW,PM,MCLRE,LPT1OSC,ECCPMX,CCP2MX,DEBUG,XINST,BBSIZ,LVP,STVREN,CP5,CP4,CP3,CP2,CP1,CP0,CPD,CPB,WRT5,WRT4,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBRT5,EBRT4,EBRT3,EBRT2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(160),
            new PicDevice("PIC18F6722", 0x800, 0x1FFFF, 0, 1023,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,WAIT,BW,ABW,PM,MCLRE,LPT1OSC,ECCPMX,CCP2MX,DEBUG,XINST,BBSIZ,LVP,STVREN,CP7,CP6,CP5,CP4,CP3,CP2,CP1,CP0,CPD,CPB,WRT7,WRT6,WRT5,WRT4,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBRT7,EBRT6,EBRT5,EBRT4,EBRT3,EBRT2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(161),
            new PicDevice("PIC18F8722", 0x800, 0x1FFFF, 0, 1023,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,WAIT,BW,ABW,PM,MCLRE,LPT1OSC,ECCPMX,CCP2MX,DEBUG,XINST,BBSIZ,LVP,STVREN,CP7,CP6,CP5,CP4,CP3,CP2,CP1,CP0,CPD,CPB,WRT7,WRT6,WRT5,WRT4,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBRT7,EBRT6,EBRT5,EBRT4,EBRT3,EBRT2,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(267),
            new PicDevice("PIC18F2221", 0x800, 0xFFF, 0, 256,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,BBSIZ,ICPRT,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(267),
            new PicDevice("PIC18F2221", 0x800, 0xFFF, 0, 256,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,BBSIZ,ICPRT,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(264),
            new PicDevice("PIC18F4321", 0x800, 0x1FFF, 0, 256,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,BBSIZ,ICPRT,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB"));
	device = new PicDevice("PIC18F2321", 0x800, 0x1FFF, 0, 256,
				           "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,BBSIZ,ICPRT,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB");
    device.alternateDevice =
            new PicDevice("PIC18F4523", 0x800, 0x7FFF, 0, 256,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB");
	deviceTable.put(new Integer(265), device);
	deviceTable.put(new Integer(266),
            new PicDevice("PIC18F2221", 0x800, 0xFFF, 0, 256,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,BBSIZ,ICPRT,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(267),
            new PicDevice("PIC18F2221", 0x800, 0xFFF, 0, 256,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,BBSIZ,ICPRT,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(269),
            new PicDevice("PIC18F4423", 0x800, 0x3FFF, 0, 256,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(277),
            new PicDevice("PIC18F2423", 0x800, 0x3FFF, 0, 256,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP1,CP0,CPD,CPB,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR1,EBTR0,EBTRB"));
	deviceTable.put(new Integer(273),
            new PicDevice("PIC18F2523", 0x800, 0x7FFF, 0, 256,
				          "IESO,FCMEN,FOSC30,BORV,BOREN10,PWRTEN,WDTPS30,WDTEN,MCLRE,LPT1OSC,PBADEN,CCP2MX,DEBUG,XINST,LVP,STVREN,CP3,CP2,CP1,CP0,CPD,CPB,WRT3,WRT2,WRT1,WRT0,WRTD,WRTB,WRTC,EBTR3,EBTR2,EBTR1,EBTR0,EBTRB"));
  } // initPicDevices

  private void writeFile(String path) {
    Properties newProps;
    FileOutputStream fileOut = null;
    Enumeration pics;
    PicDevice picDevice;
    Integer deviceId;

    newProps = new Properties();

    pics = deviceTable.keys();

    // For each device in the table, store the PIC as a property
    for (pics = deviceTable.keys(); pics.hasMoreElements(); ) {
      deviceId = (Integer) pics.nextElement();
      picDevice = (PicDevice) deviceTable.get(deviceId);
      newProps.put(picDevice.name, getDeviceString(deviceId, picDevice));
      if (picDevice.alternateDevice != null)
        newProps.put(picDevice.alternateDevice.name, getDeviceString(deviceId, picDevice.alternateDevice));
    }

    // Create the properties file
    try {
      fileOut = new FileOutputStream(path);
    }
    catch (IOException e) {
      System.out.println("Could not open file '" + path + "' for writing");
    }

    // Store the properties
    try {
      newProps.store(fileOut, "PIC devices");
    }
    catch (IOException e) {
      System.out.println("Could not store properties: " + e);
    }

    // Close properties file
    try {
      fileOut.close();
    } catch (IOException e1) {
      System.out.println("Could not close file for writing");
    }
  } // writeFile

  /**
  Load device information contained in the PIC devices property file.
  */
  public void load() {
    Properties props;
    File f = new File(PROPERTY_FILENAME);

    if (!f.exists()) {
      initPicDevices();
      f = new File(System.getProperty("user.dir") + File.separator +
                   PROPERTY_FILENAME);
    }

    if (f.exists()) {
      try {
        FileInputStream fis = new FileInputStream(f);
        props = new Properties();
        props.load(fis);
        fis.close();
        loadPicDevices(props);
      }
      catch (IOException e) {
        System.err.println(e);
      }
    }
    else {
      save();
    }
  } // load

  /**
  Fill in the PIC device table accroding to the settings in the properties object.
  */
  private void loadPicDevices(Properties props) {
    Enumeration pics;
    String deviceStr;
    PicDevice device, existingDevice;
    int deviceId;
    String name;

    // Transfer PIC information from properties object to device table.
    for (pics = props.propertyNames(); pics.hasMoreElements(); ) {
      name = (String) pics.nextElement();
      deviceStr = props.getProperty(name);
      device = new PicDevice(name, deviceStr);
      deviceId = getDeviceId(deviceStr);
      
      existingDevice = (PicDevice) deviceTable.get(new Integer(deviceId));
      if (existingDevice == null)
      	// Store device in device table.
        deviceTable.put(new Integer(deviceId), device);
      else
      	// Store device as alternate device for this specific Device ID.
      	existingDevice.alternateDevice = device;
    }
  } // loadPicDevices

  public void save() {
    writeFile(PROPERTY_FILENAME);
  } // save

  public String getDeviceString(Integer deviceId, PicDevice device) {
    return deviceId + "," + 
      Integer.toHexString(device.loProgramMem) + "," + 
      Integer.toHexString(device.hiProgramMem) + "," + 
      Integer.toHexString(device.loEeData) + "," +
      Integer.toHexString(device.hiEeData) + "," +
	  device.getConfigBitsString();
  } // getDeviceString

  public int getDeviceId(String propertyString) {
    StringTokenizer st = new StringTokenizer(propertyString, ",");

    if (!st.hasMoreTokens())
      return 0;

    return Integer.parseInt(st.nextToken());
  } // getDeviceId
} // PicDeviceProperties
