/*
 * @(#)ProgressDialog.java	0.1 02/09/27 SMI
 *
 * Copyright (c) 2002 Martin Dubuc All Rights Reserved.
 *
 * Martin Dubuc grants you ("Licensee") a non-exclusive, royalty free, 
 * license to use, modify and redistribute this software in source and
 * binary code form, provided that i) this copyright notice and license
 * appear on all copies of the software; and ii) Licensee does not utilize
 * the software in a manner which is disparaging to Martin Dubuc.
 *
 * This software is provided "AS IS," without a warranty of any kind.
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. MARTIN
 * DUBUC AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE
 * SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL MARTIN DUBUC OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR
 * DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE
 * DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,
 * ARISING OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF MARTIN
 * DUBUC HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control
 * of aircraft, air traffic, aircraft navigation or aircraft
 * communications; or in the design, construction, operation or
 * maintenance of any nuclear facility. Licensee represents and
 * warrants that it will not use or redistribute the Software for such
 * purposes.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
A single response modal progress dialog. This class is configurable for message 
and title. When the Cancel button is pressed the dialog returns.
*/
public class ProgressDialog extends JDialog implements ActionListener {
  final static long serialVersionUID = 0;
  
  private JProgressBar progressBar;
  private Bootloader bootloader;

  /**
  Creates a new <code>ProgressDialog</code> with three lines of message and
  a title.

  @param parent Any Frame.
  @param title The title to appear in the border of the dialog.
  */
  public ProgressDialog(Bootloader bootloader, 
                        String title) {
    super(bootloader, title, false);

    this.bootloader = bootloader;
    
    Container contentPane = getContentPane();

    JPanel progressBarPanel = new JPanel();
    progressBar = new JProgressBar(0, 100) {
      final static long serialVersionUID = 0;

	  public Dimension getPreferredSize() {
	    return new Dimension(280, 20);
	  }
    };
    progressBar.setValue(0);
    progressBar.setStringPainted(true);
    progressBarPanel.add(progressBar);
    contentPane.add(progressBarPanel, "North");

    Panel buttonPanel = new Panel();
    Button cancelButton = new Button(BootloaderResourceBundle.getString("CANCEL_TEXT"));
    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);
    contentPane.add(buttonPanel, "South");

    setSize(300, 90);
    setLocation(bootloader.getLocationOnScreen().x + 30, 
                bootloader.getLocationOnScreen().y + 30);
    setVisible(false);
  } // ProgressDialog

  /**
  Handles events from the Cancel button. When Cancel is pressed the dialog
  becomes invisible, disposes of its self, and returns.
  */
  public void actionPerformed(ActionEvent e) {
    setVisible(false);
    bootloader.terminateCommand(Bootloader.COMMAND_TERM_REASON_USER);
  } // actionPerformed

  public void setProgressBar(int value) {
    progressBar.setValue(value);
  } // setProgressBar
}
