/* @(#)ScrollableAction.java	0.1 02/10/03 SMI
 *
 * Copyright (c) 2002 Martin Dubuc All Rights Reserved.
 *
 * Martin Dubuc grants you ("Licensee") a non-exclusive, royalty free, license
 * to use, modify and redistribute this software in source and binary
 * code form, provided that i) this copyright notice and license appear
 * on all copies of the software; and ii) Licensee does not utilize the
 * software in a manner which is disparaging to Martin Dubuc.
 *
 * This software is provided "AS IS," without a warranty of any kind.
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. MARTIN DUBUC
 * AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE
 * SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL MARTIN DUBUC OR ITS LICENSORS
 * BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING
 * OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF MARTIN DUBUC
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control
 * of aircraft, air traffic, aircraft navigation or aircraft
 * communications; or in the design, construction, operation or
 * maintenance of any nuclear facility. Licensee represents and
 * warrants that it will not use or redistribute the Software for such
 * purposes.
 */

import javax.swing.*;
import java.awt.event.*;

/**
A class that handles the details of scrollable action.
*/
public class ScrollableAction extends AbstractAction {
  final static long serialVersionUID = 0;
  
  private JScrollPane scrollPane;
  private int orientation;
  private int type;
  private int direction;

  public static final int UNIT = 1;
  public static final int BLOCK = 2;
  public static final int AREA = 3;

  /**
  Creates a ScollableAction object and initializes variables passed in
  as params.

  @param p The scrollable panel.
  @param orient The scroll orientation.
  @param t The type.
  @param dir The direction.
  */
  public ScrollableAction(JScrollPane p, int orient, int t, int dir) {
    scrollPane = p;
    orientation = orient;
    type = t;
    direction = dir;
  } // ScrollableAction

  public void register(JScrollPane p, int orient, int t,
                       int dir, int key, int modifier) {
    p.registerKeyboardAction(new ScrollableAction(p, orient, t, dir),
                             KeyStroke.getKeyStroke(key, modifier, false),
                             JComponent.WHEN_IN_FOCUSED_WINDOW);
  } // register

  public void actionPerformed(ActionEvent evt) {
    JScrollBar scrollBar;

    if (orientation == JScrollBar.HORIZONTAL)
      scrollBar = scrollPane.getHorizontalScrollBar();
    else
      scrollBar = scrollPane.getVerticalScrollBar();

    if (scrollBar == null || !scrollBar.isVisible())
      return;

    int increment;
    if (type == UNIT) {
      increment = scrollBar.getUnitIncrement(direction);
      scrollBar.setValue(scrollBar.getValue() + increment * direction);
    }
    else if (type == BLOCK) {
      increment = scrollBar.getBlockIncrement(direction);
      scrollBar.setValue(scrollBar.getValue() + increment * direction);
    }
    else {
      if (direction < 0)
        scrollBar.setValue(scrollBar.getMinimum());
       else
         scrollBar.setValue(scrollBar.getMaximum());
    }
  } // actionPerformed
} // ScrollableAction
